<?php

/**
 * The admin-specific functionalities
 *
 * @link       https://www.myagileprivacy.com/
 * @since      1.0.12
 *
 * @package    MyAgilePrivacy
 * @subpackage MyAgilePrivacy/admin
 */

/**
 * The admin-specific functionalities
 *
 * Defines the plugin name, version, and hooks
 *
 * @package    MyAgilePrivacy
 * @subpackage MyAgilePrivacy/admin
 * @author     https://www.myagileprivacy.com/
 */
class MyAgilePrivacyAdmin {

	/**
	 * Plugin Name
	 *
	 * @since    1.0.12
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * Plugin Version
	 *
	 * @since    1.0.12
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Constructor
	 *
	 * @since    1.0.12
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version, $plugin_obj )
	{
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->set_locale();
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 * Rewritten func to ignore messy mofile
	 * @since    1.3.9
	 * @access   private
	 */
	private function my_load_plugin_textdomain( $domain, $deprecated = false, $plugin_rel_path = false ) {

		if( isset( $GLOBALS['wp_version'] ) && version_compare( $GLOBALS['wp_version'], '5.0', '<' ) )
		{
			$locale = apply_filters( 'plugin_locale', is_admin() ? get_user_locale() : get_locale(), $domain );
		}
		else
		{
			$locale = apply_filters( 'plugin_locale', determine_locale(), $domain );
		}

		$mofile = $domain . '-' . $locale . '.mo';

		/*
		// Try to load from the languages directory first.
		if ( load_textdomain( $domain, WP_LANG_DIR . '/plugins/' . $mofile ) ) {
			return true;
		}
		*/

		if ( false !== $plugin_rel_path ) {
			$path = WP_PLUGIN_DIR . '/' . trim( $plugin_rel_path, '/' );
		} elseif ( false !== $deprecated ) {
			_deprecated_argument( __FUNCTION__, '2.7.0' );
			$path = ABSPATH . trim( $deprecated, '/' );
		} else {
			$path = WP_PLUGIN_DIR;
		}

		return load_textdomain( $domain, $path . '/' . $mofile );
	}

	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * @since    1.0.12
	 * @access   private
	 */
	private function set_locale()
	{
		global $locale;

		$the_settings = MyAgilePrivacy::get_settings();

		if( isset( $the_settings['default_locale'] ) )
		{
			$old_locale = $locale;
			$locale = $the_settings['default_locale'];
		}

		$loaded = $this->my_load_plugin_textdomain(
			MAP_PLUGIN_SLUG,
			false,
			dirname( dirname( plugin_basename( __FILE__ ) ) ) . '/lang/'
		);

		if( isset( $the_settings['default_locale'] ) )
		{
			$locale = $old_locale;
		}
	}

	/**
	 * f for activating myagilepixel
	 */
	public function admin_auto_enable_cookie()
	{

		if (!function_exists('is_plugin_active')) {
			include_once(ABSPATH . 'wp-admin/includes/plugin.php');
		}

		if( is_plugin_active( 'myagilepixel/myagilepixel.php') )
		{
			$auto_activate_keys = array();

			if( defined('MAPX_my_agile_pixel_ga_on') )
			{
				$auto_activate_keys[] = 'my_agile_pixel_ga';
			}

			if( defined('MAPX_my_agile_pixel_fbq_on') )
			{
				$auto_activate_keys[] = 'my_agile_pixel_fbq';
			}

			if( defined('MAPX_my_agile_pixel_tiktok_on') )
			{
				$auto_activate_keys[] = 'my_agile_pixel_tiktok';
			}

			if( count( $auto_activate_keys ) > 0 )
	    	{
				// Get options
				$the_options = MyAgilePrivacy::get_settings();

				global $locale;
				global $sitepress;
				$is_wpml_enabled = false;
				$is_polylang_enabled = false;
				$with_multilang = false;

				if( MAP_MULTILANG_SUPPORT )
				{
					if( function_exists( 'icl_object_id' ) && $sitepress )
					{
						$is_wpml_enabled = true;
						$is_polylang_enabled = true;

						$multilang_default_lang = $sitepress->get_default_language();
						$wpml_current_lang = ICL_LANGUAGE_CODE;
						$language_list = icl_get_languages();
						$language_list_codes = array();

						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );

						foreach( $language_list as $k => $v )
						{
							if( isset( $v['code'] ) )
							{
								$language_list_codes[] = $v['code'];
							}
							elseif( isset( $v['language_code'] ) )
							{
								$language_list_codes[] = $v['language_code'];
							}
						}
					}

					if( defined( 'POLYLANG_FILE' ) )
					{
						$is_polylang_enabled = true;
						$with_multilang = true;

						$multilang_default_lang = pll_default_language();
						$language_list_codes = pll_languages_list();

						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
					}
				}



			    foreach( $auto_activate_keys as $k => $v )
			    {
	    			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $v, true );
	    			if( $v )
	    			{
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $multilang_default_lang );
						}

						$cc_args = array(
						    'posts_per_page'   	=> 	-1,
						    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
						    'meta_key'         	=> 	'_map_api_key',
						    'meta_value'       	=> 	$v,
						    'post_status' 		=> 	array( 'draft', 'publish' )
						);

						if( $is_polylang_enabled )
						{
							$cc_args['lang'] = $multilang_default_lang;
						}

						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $cc_args, true );
						wp_reset_query();
						$cc_query = new WP_Query( $cc_args );

						//wpml language reset
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $wpml_current_lang );
						}

						if ( $cc_query->have_posts() )
						{
							foreach ( $cc_query->get_posts() as $p )
							{
					        	$main_post_id = $p->ID;

								//update
								$my_post = array(
									'ID'           	=> 	$main_post_id,
									'post_status'	=>	'publish',
								);

								if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

								wp_update_post( $my_post );

								if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "published ".$v , true );

								update_post_meta( $main_post_id, "_map_auto_detected", 1 );

							    if( $with_multilang )
							    {
							    	if( $is_wpml_enabled )
							    	{
										$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );

						    			$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );

										//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $t_post_id );
					    				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );
							    	}

							    	if( $is_polylang_enabled )
							    	{
							    		$translations = pll_get_post_translations( $main_post_id );
							    	}

					    			if( $translations )
					    			{
					    				$to_update = array();

								    	foreach( $translations as $t_key => $t_value )
								    	{
								    		if( $t_key != $multilang_default_lang )
								    		{
								    			if( $is_wpml_enabled )
								    			{
								    				$to_update[ $t_key ] = $t_value->element_id;
								    			}

								    			if( $is_polylang_enabled )
								    			{
								    				$to_update[ $t_key ] = $t_value;
								    			}
								    		}
								    	}

								    	if( $to_update )
								    	{
								    		foreach( $to_update as $key_to_update => $to_update_post_id )
								    		{
								    			if( $to_update_post_id )
								    			{
													//update
													$my_post = array(
														'ID'           	=> 	$to_update_post_id,
														'post_status'	=>	'publish',
													);

													//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

											    	wp_update_post( $my_post );

											    	update_post_meta( $to_update_post_id, "_map_auto_detected", 1 );
								    			}
								    		}
								    	}
					    			}
							    }

							}
						}
						wp_reset_query();
	    			}
			    }
	    	}

		}
	}

	/**
	 * Unset some data for polylang
	 *
	 * @since    1.2.0
	 */
	public function add_cpt_to_pll( $post_types, $is_settings )
	{
	    if ( $is_settings ) {
	        // hides cpt from the list of custom post types in Polylang settings
	        unset( $post_types[ MAP_POST_TYPE_COOKIES ] );
	        unset( $post_types[ MAP_POST_TYPE_POLICY ] );
	    } else {
	        // enables language and translation management
	        $post_types[ MAP_POST_TYPE_COOKIES] =  MAP_POST_TYPE_COOKIES;
	        $post_types[ MAP_POST_TYPE_POLICY] =  MAP_POST_TYPE_POLICY;
	    }
	    return $post_types;
	}


	/**
	 * Stylesheets for the admin area.
	 *
	 * @since    1.0.12
	 * @access   public
	 */
	public function enqueue_styles()
	{
		if( isset($_GET['page']) && $_GET['page'] == MAP_POST_TYPE_COOKIES.'_settings' )
		{
			wp_enqueue_style( 'wp-color-picker' );
			wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) ."css/my-agile-privacy-admin.css", array(),$this->version, 'all' );
		}
	}

	/**
	 * Function called on custom post editing
	 *
	 * @since    1.0.12
	 * @access   public
	 */
	public function custom_post_type_editor()
	{
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) ."css/my-agile-privacy-admin.css", array(),$this->version, 'all' );
	}


	/**
	 * Js for the admin area.
	 *
	 * @since    1.0.12
	 */
	public function enqueue_scripts()
	{
		if( isset($_GET['page']) && $_GET['page'] == MAP_POST_TYPE_COOKIES.'_settings' )
		{
			wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/my-agile-privacy-admin.js', array( 'jquery' ,'wp-color-picker'), $this->version, false );
		}
	}

	/**
	 * Admin footer hook
	 * @since    1.0.12
	 * @access   public
	*/
	public function admin_footer()
	{
		if( isset( $GLOBALS['wp_version'] ) )
		{
			if( version_compare( $GLOBALS['wp_version'], '5.9', '<=' ) )
			{
				return false;
			}
		}

		if( is_admin() && !wp_doing_ajax() && empty($_POST) )
		{
			if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'admin_footer' );

			global $locale;
			global $sitepress;
			$is_wpml_enabled = false;
			$is_polylang_enabled = false;
			$with_multilang = false;

			if( MAP_MULTILANG_SUPPORT )
			{
				if( function_exists( 'icl_object_id' ) && $sitepress )
				{
					$is_wpml_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = $sitepress->get_default_language();
					$wpml_current_lang = ICL_LANGUAGE_CODE;
					$language_list = icl_get_languages();
					$language_list_codes = array();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list );

					foreach( $language_list as $k => $v )
					{
						if( isset( $v['code'] ) )
						{
							$language_list_codes[] = $v['code'];
						}
						elseif( isset( $v['language_code'] ) )
						{
							$language_list_codes[] = $v['language_code'];
						}
					}
				}

				if( defined( 'POLYLANG_FILE' ) )
				{
					$is_polylang_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = pll_default_language();
					$language_list_codes = pll_languages_list();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
				}
			}

			$do_sync = false;

			wp_reset_query();

			//check if exists post type
			$cc_args = array(
			    'posts_per_page'   => 	-1,
			    'post_type'        =>	MAP_POST_TYPE_COOKIES,
			);

			if( $is_wpml_enabled )
			{
				$cc_args['suppress_filters'] = true;
			}

			if( $is_polylang_enabled )
			{
				$cc_args['lang'] = '';
			}

			wp_reset_query();
			$cc_query = new WP_Query( $cc_args );

			if( $cc_query->have_posts() )
			{
				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'do_not_sync' );
			}
			else
			{
				$do_sync = true;
			}
			wp_reset_query();

			//check if exists post type
			$cc_args = array(
			    'posts_per_page'   => 	-1,
			    'post_type'        =>	MAP_POST_TYPE_POLICY,
			);

			if( $is_wpml_enabled )
			{
				$cc_args['suppress_filters'] = true;
			}

			if( $is_polylang_enabled )
			{
				$cc_args['lang'] = '';
			}

			wp_reset_query();
			$cc_query = new WP_Query( $cc_args );

			if( $cc_query->have_posts() )
			{
				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'do_not_sync' );
			}
			else
			{
				$do_sync = true;
			}
			wp_reset_query();

			// Get options
			$the_options = MyAgilePrivacy::get_settings();

			$now = strtotime( "now" );

			$treshold = MAP_LEGIT_SYNC_TRESHOLD;

			if( $the_options &&
				( $the_options['last_legit_sync'] == null || ( ( $the_options['last_legit_sync'] + $treshold ) < $now ) ) )
			{
				$do_sync = true;
			}

			if( $do_sync )
			{
				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'footer triggering sync_cookies_and_fixed_texts' );

				$this->sync_cookies_and_fixed_texts();
			}

			if( MAP_MULTILANG_SUPPORT )
			{
				$parity_checker = array();

				if( $is_wpml_enabled )
				{
					$language_list = icl_get_languages();

					foreach( $language_list as $k => $v )
					{
						if( isset( $v['code'] ) )
						{
							$language_list_codes[] = $v['code'];
							$sitepress->switch_lang( $v['code'] );
						}
						elseif( isset( $v['language_code'] ) )
						{
							$language_list_codes[] = $v['language_code'];
							$sitepress->switch_lang( $v['language_code'] );
						}

						$cc_args = array(
						    'posts_per_page'   	=> 	-1,
						    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
						    'post_status' 		=> 	array('publish' ),
							'meta_query'    	=> array (
								array(
									'key' 		=> '_map_api_key',
									'value' 	=> '' ,
									'compare' 	=> 'EXISTS',
									'type'    	=> 'NUMERIC',
								),
							),
						);

						wp_reset_query();
						$cc_query = new WP_Query( $cc_args );

						if( $cc_query->found_posts > 0 )
						{
							$parity_checker[] = $cc_query->found_posts;
						}

					}

					$sitepress->switch_lang( $wpml_current_lang );
				}

				if( $is_polylang_enabled )
				{
					foreach( $language_list_codes as $k => $v )
					{
						$cc_args = array(
						    'posts_per_page'   	=> 	-1,
						    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
						    'post_status' 		=> 	array('publish' ),
							'meta_query'    	=> array (
								array(
									'key' 		=> '_map_api_key',
									'value' 	=> '' ,
									'compare' 	=> 'EXISTS',
									'type'    	=> 'NUMERIC',
								),
							),
						);

						$cc_args['lang'] = $v;

						wp_reset_query();
						$cc_query = new WP_Query( $cc_args );

						if( $cc_query->found_posts > 0 )
						{
							$parity_checker[] = $cc_query->found_posts;
						}
					}
				}

				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $parity_checker );

				if( $with_multilang && count( array_unique( $parity_checker ) ) != 1 )
				{
					if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $parity_checker );

					$this->triggerInternalCookiesRefresh();
				}
			}
		}
	}

	/**
	* Function for calling remote sync via cronjob
	 * @since    1.3.7
	 * @access   public
	*/
	public function do_cron_sync_install_counter()
	{
		$data = $this->js_get_plugin_stats();

		if( isset( $data ) )
		{
			update_option( MAP_PLUGIN_STATS, $data );
		}
	}


	/**
	* Function for calling remote sync via cronjob
	 * @since    1.0.12
	 * @access   public
	*/
	public function do_cron_sync()
	{
		// Get options
		$the_options = MyAgilePrivacy::get_settings();

		$sync_result = $this->sync_cookies_and_fixed_texts();

		if( $sync_result )
		{
			$the_options['last_sync'] = strtotime( "now" );
			update_option( MAP_PLUGIN_SETTINGS_FIELD, $the_options );
			$the_options = MyAgilePrivacy::get_settings();
		}

		//log reset
		if( $the_options['scanner_log'] )
		{
			$log = substr( $the_options['scanner_log'], -200 );
			$the_options['scanner_log'] = $log;
			update_option( MAP_PLUGIN_SETTINGS_FIELD, $the_options );
			$the_options = MyAgilePrivacy::get_settings();
		}

		//learning mode auto turn to live
		if( $the_options['scan_mode'] == 'learning_mode' )
		{
			$now = time();

			if( $the_options['learning_mode_last_active_timestamp'] == null )
			{
				$the_options['learning_mode_last_active_timestamp'] = $now;
				update_option( MAP_PLUGIN_SETTINGS_FIELD, $the_options );
				$the_options = MyAgilePrivacy::get_settings();
			}
			else
			{
				$turn_to_live = false;

				if( MAP_DEV_MODE )
				{
					if( $now - $the_options['learning_mode_last_active_timestamp'] > 3600 )
					{
						$turn_to_live = true;
					}
				}
				else
				{
					if( $now - $the_options['learning_mode_last_active_timestamp'] > 604800 )
					{
						$turn_to_live = true;
					}
				}

				if( $turn_to_live )
				{
					//auto turn to live after 1 week
					$the_options['learning_mode_last_active_timestamp'] = null;
					$the_options['scan_mode'] = 'config_finished';
					$the_options['scanner_log'] = null;
					update_option( MAP_PLUGIN_SETTINGS_FIELD, $the_options );
					$the_options = MyAgilePrivacy::get_settings();
				}
			}
		}
	}

	/**
	 * Function to trigger post save on parity check failed
	 * @since    1.1.9
	 * @access   public
	*/
	public function triggerInternalCookiesRefresh()
	{
		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'start triggerInternalCookiesRefresh' );

		global $locale;
		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();
			}
		}


		//check if exists
		if( $is_wpml_enabled )
		{
			$sitepress->switch_lang( $multilang_default_lang );
		}

		$cc_args = array(
		    'posts_per_page'   	=> 	-1,
		    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
		);

		if( $is_polylang_enabled )
		{
			$cc_args['lang'] = $multilang_default_lang;
		}

		wp_reset_query();
		$cc_query = new WP_Query( $cc_args );

		//wpml language reset
		if( $is_wpml_enabled )
		{
			$sitepress->switch_lang( $wpml_current_lang );
		}

		if ( $cc_query->have_posts() )
		{
		    foreach ( $cc_query->get_posts() as $p )
		    {
		        $main_post_id = $p->ID;

		        $the_post_status = get_post_status( $main_post_id );

				//update
				$my_post = array(
					'ID'           	=> 	$main_post_id,
					'post_status'	=>	$the_post_status,
				);

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

				wp_update_post( $my_post );
		   }
		}

		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'end parity fix' );
	}


	/**
	* Function for executing remote sync
	 * @since    1.0.12
	 * @access   public
	*/
	public function sync_cookies_and_fixed_texts()
	{
		// Get options
		$the_options = MyAgilePrivacy::get_settings();

		$sync_in_progress = ( get_option( MAP_PLUGIN_SYNC_IN_PROGRESS ) ) ? get_option( MAP_PLUGIN_SYNC_IN_PROGRESS ) : 0;

		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $sync_in_progress );

		if( $sync_in_progress )
		{
			$do_block = true;

			$now = strtotime( "now" );

			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $now );
			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $the_options['last_legit_sync'] );
			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( MAP_AUTORESET_SYNC_TRESHOLD );

			if( $the_options &&
				( $the_options['last_legit_sync'] ) &&
				( $now - $the_options['last_legit_sync'] > MAP_AUTORESET_SYNC_TRESHOLD ) )
			{
				$do_block = false;

				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "unsetting block" );

			}

			if( $do_block )
			{
				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "HALT sync_cookies_and_fixed_texts already in progress" );
				return;
			}
		}

		update_option( MAP_PLUGIN_SYNC_IN_PROGRESS, 1 );

		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "START sync_cookies_and_fixed_texts" );

		global $locale;
		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;
		$prevent_actions = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $multilang_default_lang );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $wpml_current_lang );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list );

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
			}

			if( $with_multilang && !$multilang_default_lang )
			{
				$is_wpml_enabled = false;
				$is_polylang_enabled = false;
				$with_multilang = false;
				$prevent_actions = true;

				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'error on multilang' );
			}

			remove_action( 'save_post_'.MAP_POST_TYPE_COOKIES, array( $this, 'save_custom_metabox_cookies' ) );
			remove_action( 'save_post_'.MAP_POST_TYPE_POLICY, array( $this, 'save_custom_metabox_policies' ) );
		}



		if( isset( $the_options['default_locale'] ) )
		{
			$old_locale = $locale;
			$locale = $the_options['default_locale'];
		}

		$urlparts = parse_url(home_url());
		$domain = $urlparts['host'];

		$data_to_send = array(
			'action'		=>	'get_cookies_and_fixed_text',
			'software_key'	=>	MAP_SOFTWARE_KEY,
			'hash'			=>	$the_options['license_code'],
			'domain'		=>	$domain,
			'locale'		=>	$locale,
			'version'		=>	MAP_PLUGIN_VERSION,
		);

		if( $with_multilang )
		{
			$data_to_send['languages'] = implode( ',', $language_list_codes );
		}

		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $data_to_send );

		$action_result = MyAgilePrivacy::call_api( $data_to_send );

		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $action_result );

		if( $action_result )
		{
			$pa = 0;

			if( !isset( $action_result['paid_license'] ) || $action_result['paid_license'] == 0  )
			{
				$is_wpml_enabled = false;
				$is_polylang_enabled = false;
				$with_multilang = false;
			}

			//bof licensing update part
			if( $action_result['success'] && $action_result['expired_license'] == false )
			{
				$license_valid = true;
				$grace_period = false;

				if( $action_result['paid_license'] == 0 )
				{
					$license_user_status = 'Demo license';
				}
				else
				{
					if( isset( $action_result['grace_period'] ) && $action_result['grace_period'] == 1 )
					{
						$license_user_status = 'Grace period - expiring soon';
						$grace_period = true;
					}
					else
					{
						$license_user_status = 'License valid';
					}

					$pa = 1;
				}
			}
			else
			{
				$license_valid = false;
				$license_user_status = $action_result['error_msg'];
			}

			$customer_email = $action_result['customer_email'];
			$summary_text = $action_result['summary_text'];

			$the_options['license_user_status'] = $license_user_status;
			$the_options['license_valid'] = $license_valid;
			$the_options['grace_period'] = $grace_period;
			$the_options['customer_email'] = $customer_email;
			$the_options['summary_text'] = $summary_text;
			$the_options['wl'] = ( isset( $action_result['wl'] ) ) ? $action_result['wl'] : 0;
			$the_options['parse_config'] = ( isset( $action_result['parse_config'] ) ) ? $action_result['parse_config'] : null;
			$the_options['last_legit_sync'] = strtotime( "now" );
			$the_options['pa'] = $pa;

			update_option( MAP_PLUGIN_SETTINGS_FIELD, $the_options );
			update_option( MAP_PLUGIN_RCONFIG, ( isset( $action_result['rconfig'] ) ) ? $action_result['rconfig'] : null );

			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $the_options );

			//eof licensing update part

			$cookies = $action_result['cookies'];
			$fixed_text = $action_result['fixed_text'];

			if( $action_result['success'] &&
				$action_result['valid_license'] )
			{
				if( $prevent_actions == false )
				{
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $with_multilang );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $is_wpml_enabled );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $is_polylang_enabled );

					foreach( $cookies as $k => $v )
					{
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $v );

						//check if exists
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $multilang_default_lang );
						}

						$cc_args = array(
						    'posts_per_page'   	=> 	-1,
						    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
						    'meta_key'         	=> 	'_map_remote_id',
						    'meta_value'       	=> 	$v['remote_id'],
						    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired')
						);

						if( $is_polylang_enabled )
						{
							$cc_args['lang'] = $multilang_default_lang;
						}

						wp_reset_query();
						$cc_query = new WP_Query( $cc_args );

						//wpml language reset
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $wpml_current_lang );
						}

						if ( $cc_query->have_posts() )
						{
						    foreach ( $cc_query->get_posts() as $p )
						    {
						        $main_post_id = $p->ID;

						        $allow_sync = get_post_meta( $main_post_id, '_map_allow_sync', true );

						        if( $allow_sync == 1 )
						        {
						        	if( $is_polylang_enabled )
						        	{
							        	$key_to_check = $multilang_default_lang;

							    		$name_key_to_read = ( isset( $v[ 'name_'.$key_to_check ] ) ) ? $v[ 'name_'.$key_to_check ] : $v[ 'name' ];
							    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_check ] ) ) ? $v[ 'text_'.$key_to_check ] : $v[ 'text' ];
						        	}
						        	else
						        	{
						        		$name_key_to_read = $v[ 'name' ];
						        		$text_key_to_read = $v[ 'text' ];
						        	}

									//update
									$my_post = array(
										'ID'           	=> 	$main_post_id,
										'post_title'   	=> 	$name_key_to_read,
										'post_content' 	=> 	$text_key_to_read,
									);

									$the_post_status = get_post_status( $main_post_id );

									if( $the_post_status == '__expired' )
									{
										$my_post['post_status'] = 'draft';
									}

									//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

								    wp_update_post( $my_post );

								    update_post_meta( $main_post_id, "_map_name", $v['name'] );
								    update_post_meta( $main_post_id, "_map_is_free", $v['is_free'] );

								    //todo to refactor
								    //update_post_meta( $main_post_id, "_map_is_necessary", ( $v['is_necessary'] ) ? 'necessary' : 'not-necessary' );

								    update_post_meta( $main_post_id, "_map_help_url", $v['help_url'] );
								    update_post_meta( $main_post_id, "_map_api_key", $v['api_key'] );

									if( $is_polylang_enabled )
									{
										pll_set_post_language( $main_post_id, $multilang_default_lang );
									}

								    if( $with_multilang )
								    {
								    	if( $is_wpml_enabled )
								    	{
									    	$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
									    	$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );
								    	}

								    	if( $is_polylang_enabled )
								    	{
								    		$translations = pll_get_post_translations( $main_post_id );
								    	}

								    	if( $translations )
								    	{
									    	$to_update = array();
									    	$to_insert = array();

									    	foreach( $language_list_codes as $the_key )
									    	{
									    		$to_insert[ $the_key ] = null;
									    	}

									    	unset( $to_insert[ $multilang_default_lang ] );

									    	foreach( $translations as $t_key => $t_value )
									    	{
									    		if( $t_key != $multilang_default_lang )
									    		{
									    			if( $is_wpml_enabled )
									    			{
									    				$to_update[ $t_key ] = $t_value->element_id;
									    			}

									    			if( $is_polylang_enabled )
									    			{
									    				$to_update[ $t_key ] = $t_value;
									    			}

									    			unset( $to_insert[ $t_key ] );
									    		}
									    	}

									    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_insert );
									    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

									    	if( $to_insert )
									    	{
									    		if( $is_polylang_enabled )
									    		{
									    			$poly_posts = array();
									    		}

										    	foreach( $to_insert as $key_to_insert => $value_to_insert )
										    	{
										    		$name_key_to_read = ( isset( $v[ 'name_'.$key_to_insert ] ) ) ? $v[ 'name_'.$key_to_insert ] : $v[ 'name_en' ];
										    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_insert ] ) ) ? $v[ 'text_'.$key_to_insert ] : $v[ 'text_en' ];

										    		$this_post_title = $name_key_to_read;
										    		$this_post_content = $text_key_to_read;

										    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $this_post_title );

										    		if( $this_post_title && $this_post_content )
										    		{
										    			$do_insert = true;

										    			if( $is_wpml_enabled )
										    			{
															//check if exists
															if( $is_wpml_enabled )
															{
																$sitepress->switch_lang( $key_to_insert );
															}

															$cc_search_args = array(
															    'posts_per_page'   	=> 	-1,
															    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
															    'meta_key'         	=> 	'_map_remote_id',
															    'meta_value'       	=> 	$v['remote_id'],
															    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired')
															);

															wp_reset_query();
															$cc_search_query = new WP_Query( $cc_search_args );

															//wpml language reset
															if( $is_wpml_enabled )
															{
																$sitepress->switch_lang( $wpml_current_lang );
															}

															if ( $cc_search_query->have_posts() )
															{
										    					$do_insert = false;
															}
										    			}

										    			if( $do_insert )
										    			{
												    		//insert
												    		$new_post = array (
															    'post_type' 		=> 	MAP_POST_TYPE_COOKIES,
															    'post_title' 		=>  $this_post_title,
															    'post_content' 		=> 	$this_post_content,
															    'post_status' 		=> 	$the_post_status,
															    'comment_status' 	=> 	'closed',
															    'ping_status' 		=> 	'closed',
															    'meta_input'		=>	array(
																    							'_map_remote_id'		=>	$v['remote_id'],
																    							'_map_name'				=>	$v['name'],
																    							'_map_is_free'			=>	$v['is_free'],
																    							'_map_is_necessary'		=>	( $v['is_necessary'] ) ? 'necessary' : 'not-necessary',
																    							'_map_help_url'			=>	$v['help_url'],
																    							'_map_api_key'			=>	$v['api_key'],
																    							'_map_allow_sync'		=>	1,
																    						)
															);


															//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $new_post );

															$translated_post_id = wp_insert_post( $new_post );

															if( $is_wpml_enabled )
															{
																$linking_details = $sitepress->set_element_language_details( $translated_post_id, 'post_'.MAP_POST_TYPE_COOKIES, $t_post_id, $key_to_insert );

																//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $linking_details );
															}

															if( $is_polylang_enabled )
															{
													    		$poly_posts[ $key_to_insert ] = $translated_post_id;
															}
										    			}
										    		}
										    	}

										    	if( $is_polylang_enabled )
										    	{
										    		foreach( $poly_posts as $poly_language => $poly_value )
										    		{
														pll_set_post_language( $poly_value, $poly_language );
										    		}

										    		$merged = array_merge( array( $multilang_default_lang => $main_post_id ) , $poly_posts );

													pll_save_post_translations( $merged );
										    	}

									    	}

									    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

									    	if( $to_update )
									    	{
									    		foreach( $to_update as $key_to_update => $to_update_post_id )
									    		{
									    			if( $to_update_post_id )
									    			{
											    		$name_key_to_read = $v[ 'name_'.$key_to_update ];
											    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_update ] ) ) ? $v[ 'text_'.$key_to_update ] : $v[ 'text_en' ];

											    		$this_post_title = $name_key_to_read;
											    		$this_post_content = $text_key_to_read;

											    		if( $this_post_title && $this_post_content )
											    		{
															//update
															$my_post = array(
																'ID'           	=> 	$to_update_post_id,
															    'post_title' 	=>  $this_post_title,
															    'post_content' 	=> 	$this_post_content,
																'post_status'	=>	$the_post_status,
															);

															//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

													    	wp_update_post( $my_post );
											    		}

									    			}
									    		}
									    	}
								    	}
								    }
						        }
						        else
						        {
									if( get_post_status( $main_post_id ) == '__expired' )
									{
										//update
										$my_post = array(
											'ID'           	=> 	$main_post_id ,
											'post_status'	=> 'draft',
										);

										wp_update_post( $my_post );

										if( $with_multilang )
										{
											if( $is_wpml_enabled )
											{
												$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
												$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );
											}

									    	if( $is_polylang_enabled )
									    	{
									    		$translations = pll_get_post_translations( $main_post_id );
									    	}

											if( $translations )
											{
												$to_update = array();

										    	foreach( $translations as $t_key => $t_value )
										    	{
										    		if( $t_key != $multilang_default_lang )
										    		{
										    			if( $is_wpml_enabled )
										    			{
										    				$to_update[ $t_key ] = $t_value->element_id;
										    			}

										    			if( $is_polylang_enabled )
										    			{
										    				$to_update[ $t_key ] = $t_value;
										    			}

										    		}
										    	}

										    	if( $to_update )
										    	{
										    		foreach( $to_update as $key_to_update => $to_update_post_id )
										    		{
										    			if( $to_update_post_id )
										    			{
															//update
															$my_post = array(
																'ID'           	=> 	$to_update_post_id,
																'post_status'	=>	'draft',
															);

															//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

													    	wp_update_post( $my_post );
										    			}
										    		}
										    	}
											}
										}
									}

						        	update_post_meta( $main_post_id, "_map_api_key", $v['api_key'] );
						        	update_post_meta( $main_post_id, "_map_help_url", $v['help_url'] );
						        }
						    }
						}
						else
						{
							//create

				        	if( $is_polylang_enabled )
				        	{
					        	$key_to_check = $multilang_default_lang;

					    		$name_key_to_read = ( isset( $v[ 'name_'.$key_to_check ] ) ) ? $v[ 'name_'.$key_to_check ] : $v[ 'name' ];
					    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_check ] ) ) ? $v[ 'text_'.$key_to_check ] : $v[ 'text' ];
				        	}
				        	else
				        	{
				        		$name_key_to_read = $v[ 'name' ];
				        		$text_key_to_read = $v[ 'text' ];
				        	}


							$new_post = array (
							    'post_type' 		=> MAP_POST_TYPE_COOKIES,
							    'post_title' 		=> $name_key_to_read,
							    'post_content' 		=> $text_key_to_read,
							    'post_status' 		=> 'draft',
							    'comment_status' 	=> 'closed',
							    'ping_status' 		=> 'closed',
							    'meta_input'		=>	array(
							    							'_map_remote_id'		=>	$v['remote_id'],
							    							'_map_name'				=>	$v['name'],
							    							'_map_is_free'			=>	$v['is_free'],
							    							'_map_is_necessary'		=>	( $v['is_necessary'] ) ? 'necessary' : 'not-necessary',
							    							'_map_help_url'			=>	$v['help_url'],
							    							'_map_api_key'			=>	$v['api_key'],
							    							'_map_allow_sync'		=>	1,
							    						)
							);

							//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $new_post );

							$main_post_id = wp_insert_post( $new_post );

							if( $is_polylang_enabled )
							{
								pll_set_post_language( $main_post_id, $multilang_default_lang );
							}

							//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $main_post_id );

							if( $with_multilang )
							{
								if( $is_wpml_enabled )
								{
									$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
									$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );
								}

						    	if( $is_polylang_enabled )
						    	{
						    		$translations = pll_get_post_translations( $main_post_id );
						    	}

								if( $translations )
								{
							    	if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );

							    	$to_insert = array();

							    	foreach( $language_list_codes as $the_key )
							    	{
							    		$to_insert[ $the_key ] = null;
							    	}

							    	unset( $to_insert[ $multilang_default_lang ] );

							    	foreach( $translations as $t_key => $t_value )
							    	{
							    		if( $t_key != $multilang_default_lang )
							    		{
							    			unset( $to_insert[ $t_key ] );
							    		}
							    	}

							    	if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_insert );

							    	if( $to_insert )
							    	{
							    		if( $is_polylang_enabled )
							    		{
							    			$poly_posts = array();
							    		}

								    	foreach( $to_insert as $key_to_insert => $value_to_insert )
								    	{
								    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $key_to_insert );

								    		$name_key_to_read = ( isset( $v[ 'name_'.$key_to_insert ] ) ) ? $v[ 'name_'.$key_to_insert ] : $v[ 'name_en' ];
								    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_insert ] ) ) ? $v[ 'text_'.$key_to_insert ] : $v[ 'text_en' ];

								    		$this_post_title = $name_key_to_read;
								    		$this_post_content = $text_key_to_read;

								    		if( $this_post_title && $this_post_content )
								    		{
								    			$do_insert = true;

								    			if( $is_wpml_enabled )
								    			{
													//check if exists
													if( $is_wpml_enabled )
													{
														$sitepress->switch_lang( $key_to_insert );
													}

													$cc_search_args = array(
													    'posts_per_page'   	=> 	-1,
													    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
													    'meta_key'         	=> 	'_map_remote_id',
													    'meta_value'       	=> 	$v['remote_id'],
													    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired')
													);

													wp_reset_query();
													$cc_search_query = new WP_Query( $cc_search_args );

													//wpml language reset
													if( $is_wpml_enabled )
													{
														$sitepress->switch_lang( $wpml_current_lang );
													}


													if ( $cc_search_query->have_posts() )
													{
								    					$do_insert = false;
													}
								    			}

								    			if( $do_insert )
								    			{
										    		//insert
										    		$new_post = array (
													    'post_type' 		=> 	MAP_POST_TYPE_COOKIES,
													    'post_title' 		=>  $this_post_title,
													    'post_content' 		=> 	$this_post_content,
													    'post_status' 		=> 	'draft',
													    'comment_status' 	=> 	'closed',
													    'ping_status' 		=> 	'closed',
													    /**/
													    'meta_input'		=>	array(
													    							'_map_remote_id'		=>	$v['remote_id'],
													    							'_map_name'				=>	$v['name'],
													    							'_map_is_free'			=>	$v['is_free'],
													    							'_map_is_necessary'		=>	( $v['is_necessary'] ) ? 'necessary' : 'not-necessary',
													    							'_map_help_url'			=>	$v['help_url'],
													    							'_map_api_key'			=>	$v['api_key'],
													    							'_map_allow_sync'		=>	1,
													    						)
													    /**/
													);

													if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $new_post );

													$translated_post_id = wp_insert_post( $new_post );

													if( $is_wpml_enabled )
													{
														$linking_details = $sitepress->set_element_language_details( $translated_post_id, 'post_'.MAP_POST_TYPE_COOKIES, $t_post_id, $key_to_insert );

														if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $linking_details );
													}

													if( $is_polylang_enabled )
													{
											    		$poly_posts[ $key_to_insert ] = $translated_post_id;
													}
								    			}
								    		}
								    	}

								    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $poly_posts );

								    	if( $is_polylang_enabled )
								    	{
								    		foreach( $poly_posts as $poly_language => $poly_value )
								    		{
												pll_set_post_language( $poly_value, $poly_language );
								    		}

								    		$merged = array_merge( array( $multilang_default_lang => $main_post_id ), $poly_posts );

								    		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $merged );

											pll_save_post_translations( $merged );
								    	}
							    	}
								}
							}
						}

						wp_reset_query();
					}


					foreach( $fixed_text as $k => $v )
					{
						//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $v );

						//check if exists
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $multilang_default_lang );
						}

						$cc_args = array(
						    'posts_per_page'   => 	-1,
						    'post_type'        =>	MAP_POST_TYPE_POLICY,
						    'meta_key'         => 	'_map_remote_id',
						    'meta_value'       => 	$v['remote_id']
						);

						if( $is_polylang_enabled )
						{
							$cc_args['lang'] = $multilang_default_lang;
						}

						wp_reset_query();
						$cc_query = new WP_Query( $cc_args );

						//wpml language reset
						if( $is_wpml_enabled )
						{
							$sitepress->switch_lang( $wpml_current_lang );
						}

						if ( $cc_query->have_posts() )
						{
						    foreach ( $cc_query->get_posts() as $p )
						    {
						        $main_post_id = $p->ID;

						        $allow_sync = get_post_meta( $main_post_id, '_map_allow_sync', true );

						        if( $allow_sync == 1 )
						        {
						        	if( $is_polylang_enabled )
						        	{
							        	$key_to_check = $multilang_default_lang;

							    		$name_key_to_read = ( isset( $v[ 'name_'.$key_to_check ] ) ) ? $v[ 'name_'.$key_to_check ] : $v[ 'name' ];
							    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_check ] ) ) ? $v[ 'text_'.$key_to_check ] : $v[ 'text' ];
						        	}
						        	else
						        	{
						        		$name_key_to_read = $v[ 'name' ];
						        		$text_key_to_read = $v[ 'text' ];
						        	}


									//update
									$my_post = array(
										'ID'           	=> 	$main_post_id ,
										'post_title'   	=> 	$name_key_to_read,
										'post_content' 	=> 	$text_key_to_read,
									);

									$the_post_status = get_post_status( $main_post_id );

									if( get_post_status( $main_post_id ) == '__expired' )
									{
										$my_post['post_status'] = 'publish';

										$the_post_status = 'publish';
									}

									//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

								    wp_update_post( $my_post );

								    update_post_meta( $main_post_id, "_map_name", $v['name'] );
								    update_post_meta( $main_post_id, "_map_is_free", $v['is_free'] );

									if( $is_polylang_enabled )
									{
										pll_set_post_language( $main_post_id, $multilang_default_lang );
									}

								    if( $with_multilang )
								    {
								    	if( $is_wpml_enabled )
								    	{
									    	$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
									    	$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_POLICY );
									    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $t_post_id );
									    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );
								    	}

								    	if( $is_polylang_enabled )
								    	{
								    		$translations = pll_get_post_translations( $main_post_id );
								    	}

								    	if( $translations )
								    	{
									    	$to_update = array();
									    	$to_insert = array();

									    	foreach( $language_list_codes as $the_key )
									    	{
									    		$to_insert[ $the_key ] = null;
									    	}

									    	unset( $to_insert[ $multilang_default_lang ] );

									    	foreach( $translations as $t_key => $t_value )
									    	{
									    		if( $t_key != $multilang_default_lang )
									    		{
									    			if( $is_wpml_enabled )
									    			{
									    				$to_update[ $t_key ] = $t_value->element_id;
									    			}

									    			if( $is_polylang_enabled )
									    			{
									    				$to_update[ $t_key ] = $t_value;
									    			}

									    			unset( $to_insert[ $t_key ] );
									    		}
									    	}

									    	if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_insert );

									    	if( $to_insert )
									    	{
									    		if( $is_polylang_enabled )
									    		{
									    			$poly_posts = array();
									    		}

										    	foreach( $to_insert as $key_to_insert => $value_to_insert )
										    	{
										    		$name_key_to_read = $v['name'];
										    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_insert ] ) ) ? $v[ 'text_'.$key_to_insert ] : $v[ 'text_en' ];

										    		$this_post_title = $name_key_to_read;
										    		$this_post_content = $text_key_to_read;

										    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $this_post_title );

										    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $this_post_content );

										    		if( $this_post_title && $this_post_content )
										    		{
										    			$do_insert = true;

										    			if( $is_wpml_enabled )
										    			{
															//check if exists
															if( $is_wpml_enabled )
															{
																$sitepress->switch_lang( $key_to_insert );
															}

															$cc_search_args = array(
															    'posts_per_page'   	=> 	-1,
															    'post_type'        	=>	MAP_POST_TYPE_POLICY,
															    'meta_key'         	=> 	'_map_remote_id',
															    'meta_value'       	=> 	$v['remote_id'],
															    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired')
															);

															wp_reset_query();
															$cc_search_query = new WP_Query( $cc_search_args );

															//wpml language reset
															if( $is_wpml_enabled )
															{
																$sitepress->switch_lang( $wpml_current_lang );
															}

															if ( $cc_search_query->have_posts() )
															{
										    					$do_insert = false;
															}
										    			}

										    			if( $do_insert )
										    			{
												    		//insert
												    		$new_post = array (
															    'post_type' 		=> 	MAP_POST_TYPE_POLICY,
															    'post_title' 		=>  $this_post_title,
															    'post_content' 		=> 	$this_post_content,
															    'post_status' 		=> 	$the_post_status,
															    'comment_status' 	=> 	'closed',
															    'ping_status' 		=> 	'closed',
															    'meta_input'		=>	array(
															    								'_map_remote_id'		=>	$v['remote_id'],
																    							'_map_name'				=>	$v['name'],
																    							'_map_is_free'			=>	$v['is_free'],
																    							'_map_option1_ack'		=>	0,
																    							'_map_option1_on'		=>	0,
																    							'_map_allow_sync'		=>	1,
																    						)
															);

															if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $new_post );

															$translated_post_id = wp_insert_post( $new_post );

															if( $is_wpml_enabled )
															{
																$linking_details = $sitepress->set_element_language_details( $translated_post_id, 'post_'.MAP_POST_TYPE_POLICY, $t_post_id, $key_to_insert );
																//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $linking_details );
															}

															if( $is_polylang_enabled )
															{
													    		$poly_posts[ $key_to_insert ] = $translated_post_id;
															}
										    			}
										    		}
										    	}

										    	if( $is_polylang_enabled )
										    	{
										    		foreach( $poly_posts as $poly_language => $poly_value )
										    		{
														pll_set_post_language( $poly_value, $poly_language );
										    		}

										    		$merged = array_merge( array( $multilang_default_lang => $main_post_id ) , $poly_posts );

													pll_save_post_translations( $merged );
										    	}
									    	}

									    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

									    	if( $to_update )
									    	{
									    		foreach( $to_update as $key_to_update => $to_update_post_id )
									    		{
									    			if( $to_update_post_id )
									    			{
											    		$name_key_to_read = $v['name'];
											    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_update ] ) ) ? $v[ 'text_'.$key_to_update ] : $v[ 'text_en' ];

											    		$this_post_title = $name_key_to_read;
											    		$this_post_content = $text_key_to_read;

											    		if( $this_post_title && $this_post_content )
											    		{
															//update
															$my_post = array(
																'ID'           	=> 	$to_update_post_id,
															    'post_title' 	=>  $this_post_title,
															    'post_content' 	=> 	$this_post_content,
																'post_status'	=>	$the_post_status,
															);

															//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

													    	wp_update_post( $my_post );
											    		}
									    			}
									    		}
									    	}
								    	}
								    }
						        }
						        else
						        {
									if( get_post_status( $main_post_id ) == '__expired' )
									{
										//update
										$my_post = array(
											'ID'           	=> 	$main_post_id ,
											'post_status'	=> 'publish',
										);

										wp_update_post( $my_post );

										if( $with_multilang )
										{
											if( $is_wpml_enabled )
											{
												$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
												$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_POLICY );
											}

											if( $translations )
											{
												$to_update = array();

										    	foreach( $translations as $t_key => $t_value )
										    	{
										    		if( $t_key != $multilang_default_lang )
										    		{
										    			$to_update[ $t_key ] = $t_value->element_id;
										    		}
										    	}

										    	if( $to_update )
										    	{
										    		foreach( $to_update as $key_to_update => $to_update_post_id )
										    		{
										    			if( $to_update_post_id )
										    			{
															//update
															$my_post = array(
																'ID'           	=> 	$to_update_post_id,
																'post_status'	=>	'draft',
															);

															//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

													    	wp_update_post( $my_post );
										    			}
										    		}
										    	}
											}
										}
									}
						        }
						    }
						}
						else
						{
				        	if( $is_polylang_enabled )
				        	{
					        	$key_to_check = $multilang_default_lang;

					    		$name_key_to_read = ( isset( $v[ 'name_'.$key_to_check ] ) ) ? $v[ 'name_'.$key_to_check ] : $v[ 'name' ];
					    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_check ] ) ) ? $v[ 'text_'.$key_to_check ] : $v[ 'text' ];
				        	}
				        	else
				        	{
				        		$name_key_to_read = $v[ 'name' ];
				        		$text_key_to_read = $v[ 'text' ];
				        	}

							//create
							$new_post = array (
							    'post_type' 		=> 	MAP_POST_TYPE_POLICY,
							    'post_title' 		=> $name_key_to_read,
							    'post_content' 		=> $text_key_to_read,
							    'post_status' 		=> 'publish',
							    'comment_status' 	=> 'closed',
							    'ping_status' 		=> 'closed',
							    'meta_input'		=>	array(
							    							'_map_remote_id'		=>	$v['remote_id'],
							    							'_map_name'				=>	$v['name'],
							    							'_map_is_free'			=>	$v['is_free'],
							    							'_map_allow_sync'		=>	1,
							    						)
							);

							$main_post_id = wp_insert_post( $new_post );

							if( $is_polylang_enabled )
							{
								pll_set_post_language( $main_post_id, $multilang_default_lang );
							}

							if( $with_multilang )
							{
								if( $is_wpml_enabled )
								{
									$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
									$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_POLICY );
								}

						    	if( $is_polylang_enabled )
						    	{
						    		$translations = pll_get_post_translations( $main_post_id );
						    	}

								if( $translations )
								{
							    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );

							    	$to_insert = array();

							    	foreach( $language_list_codes as $the_key )
							    	{
							    		$to_insert[ $the_key ] = null;
							    	}

							    	unset( $to_insert[ $multilang_default_lang ] );

							    	foreach( $translations as $t_key => $t_value )
							    	{
							    		if( $t_key != $multilang_default_lang )
							    		{
							    			unset( $to_insert[ $t_key ] );
							    		}
							    	}

							    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_insert );

							    	if( $to_insert )
							    	{
							    		if( $is_polylang_enabled )
							    		{
							    			$poly_posts = array();
							    		}

								    	foreach( $to_insert as $key_to_insert => $value_to_insert )
								    	{
								    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $key_to_insert );

								    		$name_key_to_read = $v['name'];
								    		$text_key_to_read = ( isset( $v[ 'text_'.$key_to_insert ] ) ) ? $v[ 'text_'.$key_to_insert ] : $v[ 'text_en' ];

								    		$this_post_title = $name_key_to_read;
								    		$this_post_content = $text_key_to_read;

								    		if( $this_post_title && $this_post_content )
								    		{
									    		//insert
									    		$new_post = array (
												    'post_type' 		=> 	MAP_POST_TYPE_POLICY,
												    'post_title' 		=>  $this_post_title,
												    'post_content' 		=> 	$this_post_content,
												    'post_status' 		=> 	'publish',
												    'comment_status' 	=> 	'closed',
												    'ping_status' 		=> 	'closed',
												    /**/
												    'meta_input'		=>	array(
												    							'_map_remote_id'		=>	$v['remote_id'],
												    							'_map_name'				=>	$v['name'],
												    							'_map_is_free'			=>	$v['is_free'],
												    							'_map_allow_sync'		=>	1,
												    						)
												    /**/
												);

												//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $new_post );

												$translated_post_id = wp_insert_post( $new_post );

												if( $is_wpml_enabled )
												{
													$linking_details = $sitepress->set_element_language_details( $translated_post_id, 'post_'.MAP_POST_TYPE_POLICY, $t_post_id, $key_to_insert );

													//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $linking_details );
												}

												if( $is_polylang_enabled )
												{
										    		$poly_posts[ $key_to_insert ] = $translated_post_id;
												}

								    		}
								    	}

								    	if( $is_polylang_enabled )
								    	{
								    		foreach( $poly_posts as $poly_language => $poly_value )
								    		{
												pll_set_post_language( $poly_value, $poly_language );
								    		}

								    		$merged = array_merge( array( $multilang_default_lang => $main_post_id ) , $poly_posts );

								    		//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $merged );

											pll_save_post_translations( $merged );
								    	}
							    	}
								}
							}
						}

						wp_reset_query();
					}
				}

			}
			else
			{
				if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( 'make expire' );

				//check if exists
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $multilang_default_lang );
				}

				//cookies
				$cc_args = array(
				    'posts_per_page'  	=> 	-1,
				    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
				    'meta_key'         	=> 	'_map_is_free',
				    'meta_value'       	=> 	0,
				    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired'),
				);

				if( $is_polylang_enabled )
				{
					$cc_args['lang'] = $multilang_default_lang;
				}

				wp_reset_query();
				$cc_query = new WP_Query( $cc_args );

				//wpml language reset
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $wpml_current_lang );
				}

				if ( $cc_query->have_posts() )
				{
				    foreach ( $cc_query->get_posts() as $p )
				    {
				        $main_post_id = $p->ID;

				        if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $main_post_id );

				        //update
						$my_post = array(
							'ID'           	=> 	$main_post_id ,
							'post_status'	=>	'__expired',
						);

					    wp_update_post( $my_post );

						if( $is_polylang_enabled )
						{
							pll_set_post_language( $main_post_id, $multilang_default_lang );
						}

					    if( $with_multilang )
					    {
					    	if( $is_wpml_enabled )
					    	{
						    	$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
						    	$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );
					    	}

					    	if( $is_polylang_enabled )
					    	{
					    		$translations = pll_get_post_translations( $main_post_id );
					    	}

					    	if( $translations )
					    	{
						    	$to_update = array();

						    	foreach( $translations as $t_key => $t_value )
						    	{
						    		if( $t_key != $multilang_default_lang )
						    		{
						    			if( $is_wpml_enabled )
						    			{
						    				$to_update[ $t_key ] = $t_value->element_id;
						    			}

						    			if( $is_polylang_enabled )
						    			{
						    				$to_update[ $t_key ] = $t_value;
						    			}
						    		}
						    	}

						    	if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

						    	if( $to_update )
						    	{
						    		foreach( $to_update as $key_to_update => $to_update_post_id )
						    		{
						    			if( $to_update_post_id )
						    			{
											//update
											$my_post = array(
												'ID'           	=> 	$to_update_post_id,
											    'post_status'	=>	'__expired',
											);

											if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

									    	wp_update_post( $my_post );
						    			}
						    		}
						    	}
					    	}
					    }
				    }
				}
				wp_reset_query();

				//check if exists
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $multilang_default_lang );
				}

				//fixed texts
				$cc_args = array(
				    'posts_per_page'  	=> 	-1,
				    'post_type'        	=>	MAP_POST_TYPE_POLICY,
				    'meta_key'         	=> 	'_map_is_free',
				    'meta_value'       	=> 	0,
				    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired'),
				);

				if( $is_polylang_enabled )
				{
					$cc_args['lang'] = $multilang_default_lang;
				}

				wp_reset_query();
				$cc_query = new WP_Query( $cc_args );

				//wpml language reset
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $wpml_current_lang );
				}

				if ( $cc_query->have_posts() )
				{
				    foreach ( $cc_query->get_posts() as $p )
				    {
				        $main_post_id = $p->ID;

				        //update
						$my_post = array(
							'ID'           	=> 	$main_post_id ,
							'post_status'	=>	'__expired',
						);

					    wp_update_post( $my_post );

						if( $is_polylang_enabled )
						{
							pll_set_post_language( $main_post_id, $multilang_default_lang );
						}

					    if( $with_multilang )
					    {
					    	if( $is_wpml_enabled )
					    	{
						    	$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
						    	$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );
					    	}

					    	if( $is_polylang_enabled )
					    	{
					    		$translations = pll_get_post_translations( $main_post_id );
					    	}

					    	if( $translations )
					    	{
						    	$to_update = array();

						    	foreach( $translations as $t_key => $t_value )
						    	{
						    		if( $t_key != $multilang_default_lang )
						    		{
						    			if( $is_wpml_enabled )
						    			{
						    				$to_update[ $t_key ] = $t_value->element_id;
						    			}

						    			if( $is_polylang_enabled )
						    			{
						    				$to_update[ $t_key ] = $t_value;
						    			}
						    		}
						    	}

						    	if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

						    	if( $to_update )
						    	{
						    		foreach( $to_update as $key_to_update => $to_update_post_id )
						    		{
						    			if( $to_update_post_id )
						    			{
											//update
											$my_post = array(
												'ID'           	=> 	$to_update_post_id,
											    'post_status'	=>	'__expired',
											);

											if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $my_post );

									    	wp_update_post( $my_post );
						    			}
						    		}
						    	}
					    	}
					    }
				    }
				}
				wp_reset_query();
			}
		}
		else
		{
			//no action on connection error
		}

		wp_reset_query();

		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( "END sync_cookies_and_fixed_texts" );

		update_option( MAP_PLUGIN_SYNC_IN_PROGRESS, 0 );

		return true;
	}

	/**
	 * Import Cookie Callback
	 *
	 * @since    1.1.9
	 */
	public function import_admin_settings_form_callback()
	{
	    // Check nonce:
	    check_admin_referer( 'myagileprivacy-update-' . MAP_PLUGIN_SETTINGS_FIELD );

		if( $_FILES['the_imported_file']['error'] == UPLOAD_ERR_OK
			&& is_uploaded_file($_FILES['the_imported_file']['tmp_name']))
		{
			$file_content = file_get_contents($_FILES['the_imported_file']['tmp_name']);

			$cookies = json_decode( $file_content, true );

			global $locale;
			global $sitepress;
			$is_wpml_enabled = false;
			$is_polylang_enabled = false;
			$with_multilang = false;

			if( MAP_MULTILANG_SUPPORT )
			{
				if( function_exists( 'icl_object_id' ) && $sitepress )
				{
					$is_wpml_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = $sitepress->get_default_language();
					$wpml_current_lang = ICL_LANGUAGE_CODE;
					$language_list = icl_get_languages();
					$language_list_codes = array();

					foreach( $language_list as $k => $v )
					{
						if( isset( $v['code'] ) )
						{
							$language_list_codes[] = $v['code'];
						}
						elseif( isset( $v['language_code'] ) )
						{
							$language_list_codes[] = $v['language_code'];
						}
					}
				}

				if( defined( 'POLYLANG_FILE' ) )
				{
					$is_polylang_enabled = true;
					$with_multilang = true;

					$multilang_default_lang = pll_default_language();
					$language_list_codes = pll_languages_list();

					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
					//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
				}

				remove_action( 'save_post_'.MAP_POST_TYPE_COOKIES, array( $this, 'save_custom_metabox_cookies' ) );
			}

			foreach( $cookies['standard'] as $k => $v )
			{
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $multilang_default_lang );
				}

				$cc_args = array(
				    'posts_per_page'   	=> 	-1,
				    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
				    'meta_key'         	=> 	'_map_api_key',
				    'meta_value'       	=> 	$k,
				    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit')
				);

				if( $is_polylang_enabled )
				{
					$cc_args['lang'] = $multilang_default_lang;
				}

				if( $is_wpml_enabled )
				{
					//$cc_args['suppress_filters'] = true;
				}
				wp_reset_query();
				$cc_query = new WP_Query( $cc_args );

				//wpml language reset
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $wpml_current_lang );
				}

				if ( $cc_query->have_posts() )
				{
					foreach ( $cc_query->get_posts() as $p )
					{
				        $main_post_id = $p->ID;

				        $my_post = array(
				        	'ID'           	=> 	$main_post_id,
				        	'post_status'	=>	'publish',
				        );

					    wp_update_post( $my_post );

					    update_post_meta( $main_post_id, "_map_installation_type", $v['_map_installation_type'] );
					    update_post_meta( $main_post_id, "_map_code", $v['_map_code'] );
					    update_post_meta( $main_post_id, "_map_raw_code", $v['_map_raw_code'] );
					    update_post_meta( $main_post_id, "_map_is_necessary", $v['_map_is_necessary'] );
					    update_post_meta( $main_post_id, "_map_allow_sync", intval( $v['_map_allow_sync'] ) );
					    update_post_meta( $main_post_id, "_map_auto_detected", intval( $v['_map_auto_detected'] ) );
					    update_post_meta( $main_post_id, "_map_auto_detected_override", intval( $v['_map_auto_detected_override'] ) );
					}
				}
			}


			foreach( $cookies['custom'] as $k => $v )
			{
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $multilang_default_lang );
				}

				$new_post = array (
				    'post_type' 		=> MAP_POST_TYPE_COOKIES,
				    'post_title' 		=> $v['title'],
				    'post_content' 		=> $v['content'],
				    'post_status' 		=> 'publish',
				    'comment_status' 	=> 'closed',
				    'ping_status' 		=> 'closed',
				    'meta_input'		=>	array(
				    							'_map_installation_type'		=>	$v['_map_installation_type'],
				    							'_map_code'						=>	$v['_map_code'],
				    							'_map_raw_code'					=>	$v['_map_raw_code'],
				    							'_map_is_necessary'				=>	$v['_map_is_necessary'],
				    							'_map_allow_sync'				=>	intval( $v['_map_allow_sync'] ),
				    							'_map_auto_detected'			=>	intval( $v['_map_auto_detected'] ),
				    							'_map_auto_detected_override'	=>	intval( $v['_map_auto_detected_override'] ),
				    						)
				);

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $new_post );

				$main_post_id = wp_insert_post( $new_post );

				if( $is_polylang_enabled )
				{
					pll_set_post_language( $main_post_id, $multilang_default_lang );
				}

				//wpml language reset
				if( $is_wpml_enabled )
				{
					$sitepress->switch_lang( $wpml_current_lang );
				}
			}

		}

		header('Location: ' . $_SERVER['HTTP_REFERER'] );

		die();
	}


	/**
	 * Export Cookie Callback
	 *
	 * @since    1.1.9
	 */
	public function backup_admin_settings_form_callback()
	{
	    // Check nonce:
	    check_admin_referer( 'myagileprivacy-update-' . MAP_PLUGIN_SETTINGS_FIELD );

		$export_data = array(
			'standard'	=>	array(),
			'custom'	=>	array(),
		);

		global $locale;
		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
			}
		}

		if( $is_wpml_enabled )
		{
			$sitepress->switch_lang( $multilang_default_lang );
		}

		$cc_args = array(
		    'posts_per_page'   	=> 	-1,
		    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
		    'post_status' 		=> 	array( 'publish' )
		);

		if( $is_polylang_enabled )
		{
			$cc_args['lang'] = $multilang_default_lang;
		}

		wp_reset_query();
		$cc_query = new WP_Query( $cc_args );

		//wpml language reset
		if( $is_wpml_enabled )
		{
			$sitepress->switch_lang( $wpml_current_lang );
		}

		if ( $cc_query->have_posts() )
		{
			foreach ( $cc_query->get_posts() as $p )
			{
				$main_post_id = $p->ID;

				$all_meta = get_post_meta( $main_post_id );

				$key = $all_meta['_map_api_key'][0];

				if( $key )
				{
					$elem_to_save = array(
						'_map_installation_type'		=>	$all_meta['_map_installation_type'][0],
						'_map_code'						=>	$all_meta['_map_code'][0],
						'_map_raw_code'					=>	$all_meta['_map_raw_code'][0],
						'_map_is_necessary'				=>	$all_meta['_map_is_necessary'][0],
						'_map_allow_sync'				=>	$all_meta['_map_allow_sync'][0],
						'_map_auto_detected'			=>	$all_meta['_map_auto_detected'][0],
						'_map_auto_detected_override'	=>	$all_meta['_map_auto_detected_override'][0],
					);

					$export_data['standard'][ $key ] = $elem_to_save;
				}
				else
				{
					$elem_to_save = array(
						'title'							=>	get_the_title( $main_post_id ),
						'content'						=>	get_post_field( 'post_content', $main_post_id ),
						'_map_installation_type'		=>	$all_meta['_map_installation_type'][0],
						'_map_code'						=>	$all_meta['_map_code'][0],
						'_map_raw_code'					=>	$all_meta['_map_raw_code'][0],
						'_map_is_necessary'				=>	$all_meta['_map_is_necessary'][0],
						'_map_allow_sync'				=>	0,
						'_map_auto_detected'			=>	0,
						'_map_auto_detected_override'	=>	0,
					);

					$export_data['custom'][] = $elem_to_save;
				}
			}
		}

		$json_data = json_encode( $export_data );

		header('Content-disposition: attachment; filename=export.json');
		header('Content-type: application/json');

		echo( $json_data );

		die();
	}


	/**
	 * Update Options Callback
	 *
	 * @since    1.0.12
	 */
	public function update_admin_settings_form_callback()
	{
		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

		global $locale;
		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
			}
		}

		$license_user_status = null;
		$license_valid = null;
		$grace_period = false;
		$customer_email = null;
		$summary_text = null;

		// Get options
		$the_options = MyAgilePrivacy::get_settings();

		if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $_POST );

		$old_license_code = $the_options['license_code'];

	    // Check nonce:
	    check_admin_referer( 'myagileprivacy-update-' . MAP_PLUGIN_SETTINGS_FIELD );

		// check form submit
	    if( isset( $_POST['action'] ) && $_POST['action'] == 'update_admin_settings_form' )
	    {
	        // Check nonce:
	        check_admin_referer( 'myagileprivacy-update-' . MAP_PLUGIN_SETTINGS_FIELD );

	        foreach( $the_options as $key => $value )
	        {
	            if( isset($_POST[$key . '_field'] ) )
	            {
	                //store sanitised values only
	                $the_options[ $key ] = MyAgilePrivacy::sanitise_settings( $key, $_POST[$key . '_field'] );
	            }
			};

			if( isset( $_POST['scan_mode_field'] ) &&
				( $_POST['scan_mode_field'] == 'turned_off' || $_POST['scan_mode_field'] == 'config_finished' ) )
			{
				$the_options['scanner_log'] = null;
				update_option( MAP_PLUGIN_JS_DETECTED_FIELDS, null );
			}

			if( isset( $_POST['reset_consent'] ) )
			{
				$timestamp = time();

				$the_options['cookie_reset_timestamp'] = $timestamp;
			}

			if( isset( $_POST['reset_cookie_settings'] ) )
			{
				wp_reset_query();
				//check if exists post type
				$cc_args = array(
				    'posts_per_page'   	=> 	-1,
				    'post_type'        	=>	MAP_POST_TYPE_COOKIES,
				    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired')
				);

				if( $is_wpml_enabled )
				{
					$cc_args['suppress_filters'] = true;
				}

				if( $is_polylang_enabled )
				{
					$cc_args['lang'] = '';
				}

				wp_reset_query();
				$cc_query = new WP_Query( $cc_args );

				if( $cc_query->have_posts() )
				{
					foreach ( $cc_query->get_posts() as $p )
					{
				        $main_post_id = $p->ID;

				        if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $main_post_id );

				        wp_delete_post( $main_post_id, true );
					}
				}

				wp_reset_query();
				//check if exists post type
				$cc_args = array(
				    'posts_per_page'   	=> 	-1,
				    'post_type'        	=>	MAP_POST_TYPE_POLICY,
				    'post_status' 		=> 	array('publish', 'pending', 'draft', 'auto-draft', 'future', 'private', 'inherit', '__expired')
				);

				if( $is_wpml_enabled )
				{
					$cc_args['suppress_filters'] = true;
				}

				if( $is_polylang_enabled )
				{
					$cc_args['lang'] = '';
				}

				wp_reset_query();
				$cc_query = new WP_Query( $cc_args );

				if( $cc_query->have_posts() )
				{
					foreach ( $cc_query->get_posts() as $p )
					{
				        $main_post_id = $p->ID;

				        if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $main_post_id );

				        wp_delete_post( $main_post_id, true );
					}
				}

				wp_reset_query();

				//refresh cookies
				$this->sync_cookies_and_fixed_texts();
			}

			if( isset( $_POST['reset_settings'] ) )
			{
				$default_settings = MyAgilePrivacy::get_default_settings();

	        	$do_not_reset_array = array(
	        		'license_code',
	        		'license_user_status',
	        		'license_valid',
	        		'grace_period',
	        		'customer_email',
	        		'summary_text',
	        		'default_locale',
	        		'website_name',
	        		'identity_name',
	        		'identity_address',
	        		'identity_email',
	        		'pa',
	        		'wl',
	        	);

	        	foreach( $default_settings as $key => $value )
	        	{
	        		if( !in_array( $key, $do_not_reset_array ) )
	        		{
	        			//lets'keep the license code and other do not reset settings
	        			$the_options[ $key ] = MyAgilePrivacy::sanitise_settings( $key, $value );
	        		}
	        	}
			}

			if( isset($_POST['license_code_field'] ) )
			{
				//validation part
				$pa = 0;

				$urlparts = parse_url(home_url());
				$domain = $urlparts['host'];

				$data_to_send = array(
					'action'		=>	'validation',
					'software_key'	=>	MAP_SOFTWARE_KEY,
					'hash'			=>	sanitize_text_field( $_POST['license_code_field'] ),
					'domain'		=>	$domain,
					'version'		=>	MAP_PLUGIN_VERSION,
				);

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $data_to_send );

				$action_result = MyAgilePrivacy::call_api( $data_to_send );

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $action_result );

				if( $action_result['success'] )
				{
					$license_valid = true;
					$grace_period = false;

					if( $action_result['paid_license'] == 0 )
					{
						$license_user_status = 'Demo license';
					}
					else
					{
						if( isset( $action_result['grace_period'] ) && $action_result['grace_period'] == 1 )
						{
							$license_user_status = 'Grace period - expiring soon';
							$grace_period = true;
						}
						else
						{
							$license_user_status = 'License valid';
						}

						$pa = 1;

						if( !( isset( $rconfig ) &&
								isset( $rconfig['disable_cronjob'] ) &&
								$rconfig['disable_cronjob'] == 1 ) )
						{
							//refresh cookies
							wp_schedule_single_event( time() + 5, 'my_agile_privacy_do_cron_sync_once_day_hook' );
						}
					}
				}
				else
				{
					$license_valid = false;
					$license_user_status = $action_result['error_msg'];
				}

				$customer_email = $action_result['customer_email'];
				$summary_text = $action_result['summary_text'];

				$the_options['license_user_status'] = $license_user_status;
				$the_options['license_valid'] = $license_valid;
				$the_options['grace_period'] = $grace_period;
				$the_options['customer_email'] = $customer_email;
				$the_options['summary_text'] = $summary_text;
				$the_options['wl'] = ( isset( $action_result['wl'] ) ) ? $action_result['wl'] : 0;
				$the_options['parse_config'] = ( isset( $action_result['parse_config'] ) ) ? $action_result['parse_config'] : null;
				$the_options['pa'] = $pa;

				update_option( MAP_PLUGIN_RCONFIG, ( isset( $action_result['rconfig'] ) ) ? $action_result['rconfig'] : null );
			}

			if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $the_options );

			update_option( MAP_PLUGIN_SETTINGS_FIELD, $the_options );
			do_action( 'map_ajax_settings_update',$_POST );

			if( isset( $_POST['force_sync'] ) )
			{
				//reset sync in progress status var for forced sync
				update_option( MAP_PLUGIN_SYNC_IN_PROGRESS, 0 );

				//lang propagation
				$this->triggerInternalCookiesRefresh();

				//refresh cookies
				$this->sync_cookies_and_fixed_texts();
			}

	        $answer = array(
	        	'success'				=>	true,
	        	'license_user_status'	=>	$license_user_status,
	        	'license_valid'			=>	$license_valid,
	        	'grace_period'			=>	$grace_period,
	        	'customer_email'		=>	$customer_email,
	        	'summary_text'			=>	$summary_text,
	        );

	        wp_send_json( $answer );

	        //if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $answer );

	        die();
	    }
	}

	/**
	 * Function for admin display order fix
	 *
	 * @since    1.0.12
	 * @access   public
	 */
	public function map_order_post_type($query)
	{
		$orderby = ( isset( $_REQUEST['orderby'] ) ) ? $_REQUEST['orderby'] : null;
		$order   = ( isset( $_REQUEST['order'] ) ) ? $_REQUEST['order'] : null;

		if($query->is_admin) {

			if ($query->get('post_type') == MAP_POST_TYPE_COOKIES || $query->get('post_type') == MAP_POST_TYPE_POLICY )
			{
				if( !$orderby )
				{
					$query->set('orderby', 'title');
				}

				if( !$order )
				{
					$query->set('order', 'ASC');
				}
			}
		}
		return $query;
	}


	/**
	 * Function for fixing link (MAP_POST_TYPE_COOKIES)
	 *
	 * @since    1.0.12
	 */
	public function map_fix_view_links( $views )
	{
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/my-agile-privacy-admin.js', array( 'jquery' ,'wp-color-picker'), $this->version, false );

		unset( $views['all'] );
		unset( $views['mine'] );
		unset( $views['__expired'] );

		echo '<div id="my_agile_privacy_backend" class="cookieWrapperView postbox" style="background:#0074a2;color:#fff;margin-top:20px;"><div class="inside">'.__('Welcome to your cookie list!', 'myagileprivacy').'<br>'.__( 'Available cookies to use are under Draft. Just edit them accordingly to your neeedings, and publish in order to make them active.', 'myagileprivacy').'</div></div>';

		return $views;
	}


	/**
	 * Add Admin Pages
	 *
	 * @since    1.0.12
	 */
	public function add_admin_pages()
	{
		global $submenu;

		remove_menu_page( 'edit.php?post_type='.MAP_POST_TYPE_POLICY );

		add_submenu_page(
			'edit.php?post_type='.MAP_POST_TYPE_COOKIES,
			__('Privacy Settings','myagileprivacy'),
			__('Privacy Settings','myagileprivacy'),
			'manage_options',
			MAP_POST_TYPE_COOKIES.'_settings',
			array($this,'admin_page_html')
		);
		add_submenu_page(
			'edit.php?post_type='.MAP_POST_TYPE_COOKIES,
			__('Policies List','myagileprivacy'),
			__('Policies List','myagileprivacy'),
			'manage_options',
			'edit.php?post_type='.MAP_POST_TYPE_POLICY
		);

		//reorder settings menu
		if(isset( $submenu ) && !empty( $submenu ) && is_array( $submenu ) )
		{
			$out = array();
			$settings_menu = array();
			if( isset( $submenu['edit.php?post_type='.MAP_POST_TYPE_COOKIES] ) && is_array( $submenu['edit.php?post_type='.MAP_POST_TYPE_COOKIES ] ))
			{
				foreach( $submenu['edit.php?post_type='.MAP_POST_TYPE_COOKIES] as $k => $v )
				{
					if( $v[2] != 'post-new.php?post_type='.MAP_POST_TYPE_COOKIES )
					{
						//slug check
						if( $v[2] == MAP_POST_TYPE_COOKIES.'_settings' )
						{
							$settings_menu = $v;
						}
						else
						{
							if( $v[2] == 'edit.php?post_type='.MAP_POST_TYPE_COOKIES )
							{
								$v[2] = 'edit.php?post_type='.MAP_POST_TYPE_COOKIES.'&post_status=publish';
								$out[$k] = $v;
							}
							else
							{
								$out[$k] = $v;
							}
						}
					}
				}
				array_unshift( $out, $settings_menu );

				$submenu['edit.php?post_type='.MAP_POST_TYPE_COOKIES] = $out;
			}
		}
	}

	/**
	* Admin page html callback
	 * @since    1.0.12
	 * @access   public
	*/
	public function admin_page_html()
	{
		// check user capabilities
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		// Get options
		$the_options = MyAgilePrivacy::get_settings();
		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

		if( !( isset( $rconfig ) &&
				isset( $rconfig['disable_install_counter'] ) &&
				$rconfig['disable_install_counter'] == 1 ) )
		{
			$stats = ( get_option( MAP_PLUGIN_STATS ) ) ? get_option( MAP_PLUGIN_STATS ) : null;
		}

	    if( !empty( $_SERVER['HTTP_X_REQUESTED_WITH'] ) &&
	    	strtolower( $_SERVER['HTTP_X_REQUESTED_WITH'] )== 'xmlhttprequest' )
        {
        	exit();
        }
	 	require_once plugin_dir_path( __FILE__ ).'views/admin_page_html.php';
	}

	/**
	* Function for outputting action link ( wp plugins area)
	 * @since    1.0.12
	 * @access   public
	*/
	public function plugin_action_links( $links )
	{
		global $locale;

		$links[] = '<a href="'. get_admin_url(null,'edit.php?post_type='.MAP_POST_TYPE_COOKIES.'&page=my-agile-privacy-c_settings') .'">'.__('Settings','myagileprivacy').'</a>';
		$links[] = '<a href="https://www.myagileprivacy.com/" target="_blank">'.__('Support','myagileprivacy').'</a>';

		if( $locale == 'it_IT' )
		{
			$links[] = '<a href="https://www.myagileprivacy.com/changelog/" target="_blank">'.__('Changelog','myagileprivacy').'</a>';
		}
		return $links;
	}


	/**
	* Admin Init
	* Set locale
	* Add custom meta box for handling custom field edit
	 * @since    1.0.12
	 * @access   public
	*/
	public function admin_init_and_add_meta_box()
	{
		$this->set_locale();

	    //add metabox
	    add_meta_box("_map_remote_id",__('Cookie ID','myagileprivacy'), array($this,"metabox_map_remote_id"), MAP_POST_TYPE_COOKIES, "side", "default");
	    add_meta_box("_map_name",__('Cookie Name','myagileprivacy'), array($this,"metabox_map_name"), MAP_POST_TYPE_COOKIES, "side", "default");
	    add_meta_box("_map_is_free",__('Free Available','myagileprivacy'), array($this,"metabox_is_free"), MAP_POST_TYPE_COOKIES, "side", "default");
	    add_meta_box("_map_is_necessary",__('Is necessary?','myagileprivacy'), array($this,"metabox_map_is_necessary"), MAP_POST_TYPE_COOKIES, "side", "default");
	    add_meta_box("_map_allow_sync",__('Allow updates?','myagileprivacy'), array($this,"metabox_map_allow_sync"), MAP_POST_TYPE_COOKIES, "side", "default");
	    add_meta_box("_map_script_installation",__('Script Installation','myagileprivacy'), array($this,"metabox_script_installation"), MAP_POST_TYPE_COOKIES, "advanced", "default");

	    add_meta_box("_map_marketing",__('Marketing Policy','myagileprivacy'), array($this,"metabox_marketing"), MAP_POST_TYPE_POLICY, "advanced", "default");
	    add_meta_box("_map_allow_sync",__('Allow updates?','myagileprivacy'), array($this,"metabox_map_allow_sync"), MAP_POST_TYPE_POLICY, "side", "default");

	    //hide metabox
	    remove_meta_box('_map_remote_id', MAP_POST_TYPE_COOKIES, 'side');
	    remove_meta_box('_map_name', MAP_POST_TYPE_COOKIES, 'side');
	    remove_meta_box('_map_is_free', MAP_POST_TYPE_COOKIES, 'side');
	}

	/**
	* metabox render function
	* renders remote_id
	* @since    1.0.12
	* @access   public
	*/
	public function metabox_map_remote_id()
	{
		global $post;
		$custom = get_post_custom( $post->ID );
		$_map_remote_id = ( isset ( $custom["_map_remote_id"][0] ) ) ? $custom["_map_remote_id"][0] : '';
		?>
		<label>Cookie ID:</label>
		<input name="_map_remote_id" value="<?php echo esc_attr( $_map_remote_id ); ?>" style="width:95%;" />
		<?php
	}

	/**
	 * metabox render function
	 * render name
	 * @since    1.0.12
	 * @access   public
	*/
	public function metabox_map_name()
	{
		global $post;
		$custom = get_post_custom( $post->ID );
		$_map_name = ( isset ( $custom["_map_name"][0] ) ) ? $custom["_map_name"][0] : '';
		?>
		<label>Cookie Name:</label>
		<input name="_map_name" value="<?php echo esc_attr( $_map_name ); ?>" style="width:95%;" />
		<?php
	}

	/**
	 * metabox render function
	 * renders is_free
	 * @since    1.0.12
	 * @access   public
	*/
	public function metabox_is_free()
	{
		global $post;
		$custom = get_post_custom( $post->ID );
		$_map_is_free = ( isset ( $custom["_map_is_free"][0] ) ) ? $custom["_map_is_free"][0] : '';
		?>
		<label>Is Free:</label>
		<input name="_map_is_free" value="<?php echo esc_attr( $_map_is_free ); ?>" style="width:95%;" />
		<?php
	}

	/**
	 * metabox render function
	 * renders allow_sync
	 * @since    1.0.12
	 * @access   public
	*/
	public function metabox_map_allow_sync()
	{
		global $post;
		$custom = get_post_custom( $post->ID );

		$map_remote_id = ( isset ( $custom["_map_remote_id"][0] ) ) ? $custom["_map_remote_id"][0] : '';

		if( $map_remote_id ):

		$selected = ( isset ( $custom["_map_allow_sync"][0] ) ) ? $custom["_map_allow_sync"][0] : '';

		?>
        <label for="_map_allow_sync"><?php _e("Allow automatic updates?", 'myagileprivacy')?>: </label>
        <br>
        <select name='_map_allow_sync' id='_map_allow_sync'>
            <?php $post_types=array(
            	'1'	=>	'Yes',
            	'0'	=> 	'No',
            );
            foreach ($post_types as $key=>$post_type): ?>
            <option value="<?php echo esc_attr($key); ?>" <?php if( $selected == esc_attr($key) ):?>selected<?php endif;?>><?php echo esc_html($post_type); ?></option>
            <?php endforeach; ?>
        </select>
        <p><?php _e("If enabled, this will auto update this element text. Disable it if you would to change the text and keep it unchanged.", 'myagileprivacy');?></p>
        <?php
    	else:
    	?>
    	<p><?php _e("This setting is unavailable on custom added cookie", 'myagileprivacy');?></p>
    	<?php
    	endif;
	}


	/**
	 * metabox render function
	 * renders is_necessary
	 * @since    1.0.12
	 * @access   public
	*/
	public function metabox_map_is_necessary()
	{
		global $post;
		$custom = get_post_custom( $post->ID );

		$selected = ( isset ( $custom["_map_is_necessary"][0] ) ) ? $custom["_map_is_necessary"][0] : '';

		?>
        <label for="_map_is_necessary"><?php _e("Is necessary?", 'myagileprivacy') ?>: </label>
        <br>
        <select name='_map_is_necessary' id='_map_is_necessary'>
            <?php $post_types=array('necessary', 'not-necessary');
            foreach ($post_types as $post_type): ?>
            <option value="<?php echo esc_attr($post_type); ?>" <?php if( $selected == esc_attr($post_type) ):?>selected<?php endif;?>><?php echo esc_html($post_type); ?></option>
            <?php endforeach; ?>
        </select>
        <?php
	}

	/**
	 * metabox render function
	 * renders marketing options
	 * @since    1.0.12
	 * @access   public
	*/
	public function metabox_marketing()
	{
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/my-agile-privacy-admin.js', array( 'jquery' ,'wp-color-picker'), $this->version, false );

		global $post;
		$custom = get_post_custom( $post->ID );

		$_map_remote_id = $custom["_map_remote_id"][0];
		$_map_option1_ack = ( isset ( $custom["_map_option1_ack"][0] ) ) ? $custom["_map_option1_ack"][0] : 0;
		$_map_option1_on = ( isset ( $custom["_map_option1_on"][0] ) ) ? $custom["_map_option1_on"][0] : 0;

		if( $_map_remote_id == 'personal_data_policy'):

		?>

		<div id="my_agile_privacy_backend" class="policyWrapperEdit">

			<label><?php _e("Compliance with Marketing Consent", 'myagileprivacy'); ?></label>
			<br>
			<br>
			<label><input name="_map_option1_ack" type="checkbox" class="hideShowInput" value="1" id="_map_option1_ack" <?php if( $_map_option1_ack == 1)echo 'checked'; ?> data-hide-show-ref="map_option1_on_wrapper">
				<?php _e("I declare that I have read, understood and adapted the forms on my site according to the specifications",'myagileprivacy');?> <a target="blank" href="https://www.myagileprivacy.com/come-essere-a-norma-con-i-moduli-di-contatto-per-le-attivita-di-marketing/"><?php _e("described here",'myagileprivacy');?></a></label>

			<div class="map_option1_on_wrapper displayNone" data-value="1">
				<br>
				<label><input name="_map_option1_on" type="checkbox" value="1" id="_map_option1_on" <?php if( $_map_option1_on == 1)echo 'checked'; ?> ><?php _e("I request consent for functional marketing activities including the sending of newsletters, paper mail, sms, mms and messaging apps.", 'myagileprivacy'); ?></label>
			</div>
		</div>

		<?php
		else:
			echo "<style>#_map_marketing.postbox{display:none;}</style>";
		endif;
	}


	/**
	 * metabox render function
	 * renders installation method
	 * @since    1.0.12
	 * @access   public
	*/
	public function metabox_script_installation()
	{
		wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/my-agile-privacy-admin.js', array( 'jquery' ,'wp-color-picker'), $this->version, false );

		global $post;
		$custom = get_post_custom( $post->ID );

		$_map_auto_detected = ( isset ( $custom["_map_auto_detected"][0] ) ) ? $custom["_map_auto_detected"][0] : 0;
		$_map_auto_detected_override = ( isset ( $custom["_map_auto_detected_override"][0] ) ) ? $custom["_map_auto_detected_override"][0] : 0;

		$_map_installation_type = ( isset ( $custom["_map_installation_type"][0] ) ) ? $custom["_map_installation_type"][0] : '';
		$_map_code = ( isset ( $custom["_map_code"][0] ) ) ? $custom["_map_code"][0] : '';
		$_map_noscript = ( isset ( $custom["_map_noscript"][0] ) ) ? $custom["_map_noscript"][0] : '';
		$_map_raw_code = ( isset ( $custom["_map_raw_code"][0] ) ) ? $custom["_map_raw_code"][0] : '';
		$_map_help_url = ( isset ( $custom["_map_help_url"][0] ) ) ? $custom["_map_help_url"][0] : '';
		$_map_api_key = ( isset ( $custom["_map_api_key"][0] ) ) ? $custom["_map_api_key"][0] : '';

		?>

		<div id="my_agile_privacy_backend">

		<?php

			$addedClass = '';
			if( $_map_auto_detected ):

			$addedClass = 'displayNone';
		?>

			<div>
				<?php _e("Warning: this cookie has been detected automatically and usually you shouldn't need to install other code.", 'myagileprivacy') ?>
			</div>
			<div>
				<label><input class="hideShowInput"
					type="checkbox"
					name="_map_auto_detected_override"
					value="1"
					data-hide-show-ref="map_auto_detected_override_wrapper"
					<?php if ($_map_auto_detected_override == 1) echo 'checked';?>
					><?php _e("Add custom code anyway.", 'myagileprivacy') ?></label>
			</div>
			<br>

		<?php
			endif;
		?>

			<div class="cookieWrapperEdit map_auto_detected_override_wrapper <?php echo $addedClass;?>" data-value="1">

				<label><?php _e("Script Installation type", 'myagileprivacy') ?></label>
				<br>
				<br>

		        <select id="_map_installation_type" name="_map_installation_type" class="hideShowInput" style="max-width:100%;" data-hide-show-ref="map_installation_type_wrapper">
		            <?php

		            $valid_options = array(

		            	'js_noscript'					=>	array(  'label' => __('Javascript-Noscript code', 'myagileprivacy'),
		            											'selected' => false ),
		            	'raw_code'						=>	array(  'label' => __('Raw code', 'myagileprivacy'),
		            											'selected' => false ),
		            );

		            $selected_value = $_map_installation_type;

		            if( isset( $valid_options[ $_map_installation_type ] ) )
		            {
		            	$valid_options[ $selected_value ]['selected'] = true;
		            }

		            foreach( $valid_options as $key => $data )
		            {
		            	if( $data['selected'] )
		            	{
		                	?>
		                	<option value="<?php echo esc_attr($key)?>" selected><?php echo esc_attr($data['label'])?></option>
		                	<?php
		            	}
		            	else
		            	{
		                	?>
		                	<option value="<?php echo esc_attr($key)?>"><?php echo esc_attr($data['label'])?></option>
		                	<?php
		            	}
		            }

		            ?>
		        </select>

				<?php
					if( $_map_api_key ):
				?>

				<p>
					 <?php
					 _e("The API KEY for this cookie is", 'myagileprivacy');
					 echo ' <b>'.esc_html( $_map_api_key ).'</b>.<br>';
					 _e("You will need this only for advanced installation", 'myagileprivacy');
					 ?>.
				</p>

				<?php
					endif;
				?>

				<?php
					if( $_map_help_url ):
				?>

				<p>
					<a target="blank" href="<?php echo esc_attr($_map_help_url);?>"><?php _e("Need help ? See this cookie installation guide", 'myagileprivacy') ?></a>
				</p>

				<?php
					endif;
				?>


		        <div class="map_installation_type_wrapper displayNone" data-value="js_noscript">

					<label><?php _e("Custom Javascript Code", 'myagileprivacy') ?></label>
					<br>

					<textarea name="_map_code" style="width:100%;height:80px;"><?php echo esc_attr( $_map_code ); ?></textarea>


					<label><?php _e("Noscript Code", 'myagileprivacy') ?></label>
					<br>

					<textarea name="_map_noscript" style="width:100%;height:80px;"><?php echo esc_attr( $_map_noscript ); ?></textarea>

				</div>

				<div class="map_installation_type_wrapper displayNone" data-value="raw_code">

					<label><?php _e("Raw Html Code", 'myagileprivacy') ?></label>
					<br>

					<textarea name="_map_raw_code" style="width:100%;height:80px;"><?php echo esc_attr( $_map_raw_code ); ?></textarea>
				</div>
			</div>

		</div>
		<?php
	}


	/**
	 * function for custom metabox save (cookies)
	 * @since    1.0.12
	 * @access   public
	*/
	public function save_custom_metabox_cookies( $main_post_id )
	{
		global $post;

		if ( isset ( $_POST["_map_remote_id"] ) )
		{
			update_post_meta( $main_post_id, "_map_remote_id", sanitize_text_field( $_POST["_map_remote_id"] ) );
	    }

	    if ( isset ( $_POST["_map_name"] ) )
	    {
			update_post_meta( $main_post_id, "_map_name", sanitize_text_field( $_POST["_map_name"] ) );
		}

	    if ( isset ( $_POST["_map_installation_type"] ) )
	    {
			update_post_meta( $main_post_id, "_map_installation_type", sanitize_text_field( $_POST["_map_installation_type"] ) );
		}

	    if ( isset ( $_POST["_map_code"] ) )
	    {
			update_post_meta( $main_post_id, "_map_code", sanitize_textarea_field( $_POST["_map_code"] ) );
		}

	    if ( isset ( $_POST["_map_raw_code"] ) )
	    {
			update_post_meta( $main_post_id, "_map_raw_code", esc_html( $_POST["_map_raw_code"] ) );
		}

	    if ( isset ( $_POST["_map_noscript"] ) )
	    {
			update_post_meta( $main_post_id, "_map_noscript", esc_html( $_POST["_map_noscript"] ) );
		}

	    if ( isset ( $_POST["_map_is_free"] ) )
	    {
			update_post_meta( $main_post_id, "_map_is_free", sanitize_text_field( $_POST["_map_is_free"] ) );
		}

	    if ( isset ( $_POST["_map_is_necessary"] ) )
	    {
			update_post_meta( $main_post_id, "_map_is_necessary", sanitize_text_field( $_POST["_map_is_necessary"] ) );
		}

	    if ( isset ( $_POST["_map_allow_sync"] ) )
	    {
			update_post_meta( $main_post_id, "_map_allow_sync", sanitize_text_field( $_POST["_map_allow_sync"] ) );
		}

	    if ( isset ( $_POST["_map_auto_detected_override"] ) )
	    {
			update_post_meta( $main_post_id, "_map_auto_detected_override", 1 );
		}
		else
		{
			update_post_meta( $main_post_id, "_map_auto_detected_override", 0 );
		}

		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
			}
		}


		if( $with_multilang )
		{
			$all_meta = get_post_meta( $main_post_id );

			//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $all_meta );

			remove_action( 'save_post_'.MAP_POST_TYPE_COOKIES, array( $this, 'save_custom_metabox_cookies' ) );

			if( $is_wpml_enabled )
			{
				$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $t_post_id );

				$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_COOKIES );

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $translations );
			}

	    	if( $is_polylang_enabled )
	    	{
	    		$translations = pll_get_post_translations( $main_post_id );
	    	}

			if( $translations )
			{
				$to_update = array();

		    	foreach( $translations as $t_key => $t_value )
		    	{
		    		if( $t_key != $multilang_default_lang )
		    		{
		    			if( $is_wpml_enabled )
		    			{
		    				$to_update[ $t_key ] = $t_value->element_id;
		    			}

		    			if( $is_polylang_enabled )
		    			{
		    				$to_update[ $t_key ] = $t_value;
		    			}
		    		}

		    	}

		    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

		    	if( $to_update )
		    	{
		    		$the_post_status = get_post_status( $main_post_id );

		    		foreach( $to_update as $key_to_update => $to_update_post_id )
		    		{
		    			if( $to_update_post_id )
		    			{
							//update
							$my_post = array(
								'ID'           	=> 	$to_update_post_id,
								'post_status'	=>	$the_post_status,
							);

					    	wp_update_post( $my_post );

					    	if( isset( $all_meta["_map_remote_id"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_remote_id", sanitize_text_field( $all_meta["_map_remote_id"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_name"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_name", sanitize_text_field( $all_meta["_map_name"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_installation_type"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_installation_type", sanitize_text_field( $all_meta["_map_installation_type"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_code"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_code", sanitize_textarea_field( $all_meta["_map_code"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_raw_code"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_raw_code", esc_html( $all_meta["_map_raw_code"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_noscript"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_noscript", esc_html( $all_meta["_map_noscript"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_is_free"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_is_free", sanitize_text_field( $all_meta["_map_is_free"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_is_necessary"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_is_necessary", sanitize_text_field( $all_meta["_map_is_necessary"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_allow_sync"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_allow_sync", sanitize_text_field( $all_meta["_map_allow_sync"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_auto_detected"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_auto_detected", $all_meta["_map_auto_detected"][0] );
					    	}

					    	if( isset( $all_meta["_map_auto_detected_override"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_auto_detected_override", $all_meta["_map_auto_detected_override"][0] );
					    	}

		    			}
		    		}
		    	}
			}

	    	add_action( 'save_post_'.MAP_POST_TYPE_COOKIES, array( $this, 'save_custom_metabox_cookies' ) );
		}
	}

	/**
	 * function for custom metabox save (policies)
	 * @since    1.0.13
	 * @access   public
	*/
	public function save_custom_metabox_policies( $main_post_id )
	{
		$rconfig = ( get_option( MAP_PLUGIN_RCONFIG ) ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

		global $post;

		$custom = get_post_custom( $post->ID );

		if ( isset ( $_POST["_map_remote_id"] ) )
		{
			update_post_meta( $main_post_id, "_map_remote_id", sanitize_text_field( $_POST["_map_remote_id"] ) );
	    }

	    if ( isset ( $_POST["_map_name"] ) )
	    {
			update_post_meta( $main_post_id, "_map_name", sanitize_text_field( $_POST["_map_name"] ) );
		}

	    if ( isset ( $_POST["_map_is_free"] ) )
	    {
			update_post_meta( $main_post_id, "_map_is_free", sanitize_text_field( $_POST["_map_is_free"] ) );
		}

	    if ( isset ( $_POST["_map_allow_sync"] ) )
	    {
			update_post_meta( $main_post_id, "_map_allow_sync", sanitize_text_field( $_POST["_map_allow_sync"] ) );
		}


		$_map_option1_ack = null;
		$_map_option1_on = null;

		if( $custom["_map_remote_id"][0] == 'personal_data_policy' )
		{
		    if ( isset ( $_POST["_map_option1_ack"] ) )
		    {
				$_map_option1_ack = 1;
			}
			else
			{
				$_map_option1_ack = 0;
			}

			update_post_meta( $main_post_id, "_map_option1_ack", $_map_option1_ack );


		    if ( isset ( $_POST["_map_option1_on"] ) )
		    {
				$_map_option1_on = 1;
			}
			else
			{
				$_map_option1_on = 0;
			}

			update_post_meta( $main_post_id, "_map_option1_on", $_map_option1_on );

			if( !( isset( $rconfig ) &&
					isset( $rconfig['disable_cronjob'] ) &&
					$rconfig['disable_cronjob'] == 1 ) )
			{
				//refresh cookies
				wp_schedule_single_event( time() + 5, 'my_agile_privacy_do_cron_sync_once_day_hook' );
			}
		}

		global $sitepress;
		$is_wpml_enabled = false;
		$is_polylang_enabled = false;
		$with_multilang = false;

		if( MAP_MULTILANG_SUPPORT )
		{
			if( function_exists( 'icl_object_id' ) && $sitepress )
			{
				$is_wpml_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = $sitepress->get_default_language();
				$wpml_current_lang = ICL_LANGUAGE_CODE;
				$language_list = icl_get_languages();
				$language_list_codes = array();

				foreach( $language_list as $k => $v )
				{
					if( isset( $v['code'] ) )
					{
						$language_list_codes[] = $v['code'];
					}
					elseif( isset( $v['language_code'] ) )
					{
						$language_list_codes[] = $v['language_code'];
					}
				}
			}

			if( defined( 'POLYLANG_FILE' ) )
			{
				$is_polylang_enabled = true;
				$with_multilang = true;

				$multilang_default_lang = pll_default_language();
				$language_list_codes = pll_languages_list();

				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $default );
				//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $language_list_codes );
			}
		}

		if( $with_multilang )
		{
			$all_meta = get_post_meta( $main_post_id );

			remove_action( 'save_post_'.MAP_POST_TYPE_POLICY, array( $this, 'save_custom_metabox_policies' ) );

			if( $is_wpml_enabled )
			{

				$t_post_id = apply_filters( 'wpml_element_trid', null, $main_post_id );
				$translations = apply_filters( 'wpml_get_element_translations', null, $t_post_id, 'post_'.MAP_POST_TYPE_POLICY );
			}

	    	if( $is_polylang_enabled )
	    	{
	    		$translations = pll_get_post_translations( $main_post_id );
	    	}

			if( $translations )
			{
				$to_update = array();

		    	foreach( $translations as $t_key => $t_value )
		    	{
		    		if( $t_key != $multilang_default_lang )
		    		{
		    			if( $is_wpml_enabled )
		    			{
		    				$to_update[ $t_key ] = $t_value->element_id;
		    			}

		    			if( $is_polylang_enabled )
		    			{
		    				$to_update[ $t_key ] = $t_value;
		    			}
		    		}
		    	}

		    	//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $to_update );

		    	if( $to_update )
		    	{
		    		$the_post_status = get_post_status( $main_post_id );

		    		foreach( $to_update as $key_to_update => $to_update_post_id )
		    		{
		    			if( $to_update_post_id )
		    			{
							//update
							$my_post = array(
								'ID'           	=> 	$to_update_post_id,
								'post_status'	=>	$the_post_status,
							);

					    	wp_update_post( $my_post );

					    	if( isset( $all_meta["_map_remote_id"][0] ) )
					    	{
					    		update_post_meta( $to_update_post_id, "_map_remote_id", sanitize_text_field( $all_meta["_map_remote_id"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_name"][0] ) )
					    	{
					    		update_post_meta( $to_update_post_id, "_map_name", sanitize_text_field( $all_meta["_map_name"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_is_free"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_is_free", sanitize_text_field( $all_meta["_map_is_free"][0] ) );
					    	}

					    	if( isset( $all_meta["_map_allow_sync"][0] ) )
					    	{
								update_post_meta( $to_update_post_id, "_map_allow_sync", sanitize_text_field( $all_meta["_map_allow_sync"][0] ) );
					    	}

					    	if( $_POST["_map_remote_id"] == 'personal_data_policy' )
					    	{
					    		update_post_meta( $to_update_post_id, "_map_option1_ack", $_map_option1_ack );
					    		update_post_meta( $to_update_post_id, "_map_option1_on", $_map_option1_on );
					    	}
		    			}
		    		}
		    	}
			}
	    	add_action( 'save_post_'.MAP_POST_TYPE_POLICY, array( $this, 'save_custom_metabox_policies' ) );
		}
	}


	/**
	 * function for defining column names ( custom post type )
	 * @since    1.0.12
	 * @access   public
	*/
	public function manage_cookies_edit_columns( $columns )
	{
		$columns = array(
			"cb" 				=> "<input type=\"checkbox\" />",
			"title"				=> __('Cookie Name','myagileprivacy'),
			"auto_detected"		=> __('Auto Detected','myagileprivacy'),
			"help"				=> __('Help link','myagileprivacy'),
			"necessary"			=> __('Is necessary?','myagileprivacy'),
			"allow_sync"		=> __("Allow automatic updates?", 'myagileprivacy'),
			"installation_type"	=> __('Script Installation','myagileprivacy'),

			//"code"   			=> "code"
		);
		return $columns;
	}


	/**
	 * function for getting column data ( custom post type )
	 * @since    1.0.12
	 * @access   public
	*/
	public function manage_cookies_posts_custom_columns( $column, $post_id=0 )
	{
		global $post;
		$custom = get_post_custom();
		switch ( $column )
		{
			case "installation_type":
				if ( isset ( $custom["_map_installation_type"][0] ) ) {

					switch ( $custom["_map_installation_type"][0] )
					{
						case 'js_noscript':
							echo __('Javascript-Noscript code', 'myagileprivacy');
						break;
						case 'raw_code':
							echo __('Raw code', 'myagileprivacy');
						break;
					}
				}
				else
				{
					echo __('Javascript-Noscript code', 'myagileprivacy');
				}
				break;

			case "auto_detected":
				if ( isset ( $custom["_map_auto_detected"] ) &&
					$custom["_map_auto_detected"][0] == 1 )
				{
					echo "<span style='color:#3fe63f;' title='Yes'>&#11044;</span>";
				}
				else
				{
					echo "<span style='color:#d0d0d0;' title='No'>&#11044;</span>";
				}
				break;


			case "help":
				if ( isset ( $custom["_map_help_url"][0] ) ) {
					echo "<a href='".esc_attr( $custom["_map_help_url"][0] )."' target='blank'>&#x1F6C8;</a>";
				}
				break;


			case "code":
				if ( isset ( $custom["_map_code"][0] ) ) {
					echo esc_html( $custom["_map_code"][0] );
				}
				elseif( isset ( $custom["_map_raw_code"][0] ) ) {
					echo esc_html( $custom["_map_raw_code"][0] );
				}
				break;

			case "necessary":
				if ( isset ( $custom["_map_is_necessary"][0] ) ) {
					echo esc_html( $custom["_map_is_necessary"][0] );
				}
				break;

			case "allow_sync":
				if ( isset ( $custom["_map_allow_sync"][0] ) ) {
					if( $custom["_map_allow_sync"][0] )
					{
						?>Yes<?php
					}
					else
					{
						?>No<?php
					}
				}
				else
				{
					?>-<?php
				}
				break;
		}
	}


	/**
	 * function for defining column names ( custom post type )
	 * @since    1.0.12
	 * @access   public
	*/
	public function manage_policies_edit_columns( $columns )
	{
		$columns = array(
			"cb" 			=> "<input type=\"checkbox\" />",
			"title"			=> "Policy",
			"allow_sync"	=> __("Allow automatic updates?", 'myagileprivacy'),
		);
		return $columns;
	}


	/**
	 * function for getting column data ( custom post type )
	 * @since    1.0.12
	 * @access   public
	*/
	public function manage_policies_posts_custom_columns( $column, $post_id=0 )
	{
		global $post;
		$custom = get_post_custom();

		switch ( $column )
		{
			case "allow_sync":
				if ( isset ( $custom["_map_allow_sync"][0] ) ) {
					if( $custom["_map_allow_sync"][0] )
					{
						?>Yes<?php
					}
					else
					{
						?>No<?php
					}
				}
				else
				{
					?>-<?php
				}
				break;
		}
	}

	/**
	 * inline help text after editor
	 * @since    1.0.12
	 * @access   public
	*/
	public function inline_help_text_after_editor()
	{
	    global $my_admin_page;
	    $screen = get_current_screen();

	    if ( is_admin() && ($screen->id == MAP_POST_TYPE_POLICY ) )
	    {
	        function add_content_after_editor() {
	            global $post;
	            $the_id = $post->ID;

	            $_map_name = get_post_meta( $the_id, '_map_name', true );

	            if( $_map_name )
	            {
					switch( $_map_name )
					{
					    case 'cookie_policy':
					        $shortcode = '[myagileprivacy_fixed_text text="cookie_policy"]';
					        break;
					    case 'personal_data_policy':
					        $shortcode = '[myagileprivacy_fixed_text text="personal_data_policy"]';
					        break;
					}

		            ?><div class="postbox" style="background:#0074a2;color:#fff;margin-top:20px;"><div class="inside"><?php
		            echo esc_html__("Please add the following shortcode to the right page:", 'myagileprivacy').'<br>';
		            echo wp_kses($shortcode, MyAgilePrivacy::allowed_html_tags() ).'<br>';
		            echo esc_html__("Remember also associate in the Privacy Settings -> Policies tab.", 'myagileprivacy');

		            ?></div></div><?php
	            }

	        }
	        add_action( 'edit_form_after_title', 'add_content_after_editor' );
	    }
	}


	/**
	 * download plugin stats
	 * @since    1.3.7
	 * @access   public
	*/
	public function js_get_plugin_stats()
	{
	    $args = (object) array(
	    	'slug' 		=> MAP_PLUGIN_SLUG,
			'fields'	=> array(
							'active_installs'	=> true,
							'downloaded'		=> false,
							'rating'			=> false,
							'description'		=> false,
							'short_description' => false,
							'donate_link'		=> false,
							'tags'				=> false,
							'sections'			=> false,
							'homepage'			=> false,
							'added'				=> false,
							'last_updated'		=> false,
							'compatibility'		=> false,
							'tested'			=> false,
							'requires'			=> false,
							'downloadlink'		=> false,
					)
		);

	    $request = array(
	    	'action' => 'plugin_information',
	    	'timeout' => 15,
	    	'request' => serialize( $args )
		);

	    $url = 'http://api.wordpress.org/plugins/info/1.0/';

	    $response = wp_remote_post( $url, array( 'body' => $request ) );

	    $plugin_info = unserialize( $response['body'] );

	    return $plugin_info;
	}
}