<?php

$defaults = MyAgilePrivacy::get_default_settings();
$settings = wp_parse_args( MyAgilePrivacy::get_settings(), $defaults );
$rconfig = get_option( MAP_PLUGIN_RCONFIG ) ? get_option( MAP_PLUGIN_RCONFIG ) : array();

// disable indexing
echo "<!--googleoff: all-->";

//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $settings );
//if( MAP_DEBUGGER ) MyAgilePrivacy::write_log( $rconfig );

$always_enable_text = __('Always Enabled', 'myagileprivacy');
$is_enabled_text = __('Enabled', 'myagileprivacy');
$is_disabled_text = __('Disabled', 'myagileprivacy');
$blocked_content_text = __('Warning: some page functionalities could not work due to your privacy choices', 'myagileprivacy');

$color_style = ( $settings['text'] != "" ) ? 'color:'.$settings['text'] : '';
$background_style = $settings['background'] != "" ? 'background-color:'.$settings['background'] : '';
$composed_style = implode( ';', array( $color_style, $background_style  ) );

$position_class = ( $settings['is_bottom'] == false ) ? 'isTop' : 'isBottom';
$with_css_effects_class = ( $settings['with_css_effects'] == true ) ? 'withEffects' : '';
$composed_class = implode( ' ', array( $position_class, $with_css_effects_class ) );

//esc_attr
$notify_message = nl2br( $the_options['notify_message'] );

if( $reload_translation )
{
	$notify_message = addslashes ( __('<div class="map-area-container"><div class="map_notification-message">This website uses technical and profiling cookies. Clicking on "Accept" authorises all profiling cookies. Clicking on "Refuse" or the X will refuse all profiling cookies. By clicking on "Customise" you can select which profiling cookies to activate.[myagileprivacy_extra_info]</div><div class="map_notification_container">[myagileprivacy_cookie_accept][myagileprivacy_cookie_reject][myagileprivacy_cookie_customize]</div></div>', 'myagileprivacy') );
}

//apply shortcodes
$notify_message = do_shortcode( stripslashes( $notify_message ) );

$bar_heading_text = trim( stripslashes( $the_options['bar_heading_text'] ) );

$notify_html = '<div id="my-agile-privacy-notification-area" class="'.esc_attr( $composed_class ).'" data-nosnippet="true" style="'.esc_attr( $composed_style ).'">' .
				'<div class="map-closebutton-right"><a class="map-button map-reject-button" data-map_action="reject"></a></div>'.
				($bar_heading_text != "" ? '<h5 class="map-head">' . esc_html( $bar_heading_text ) . '</h5>' : '').
				 $notify_message . '</div>';

$cookies_categories_data = $this->get_cookie_categories_description( false );

$disable_logo = $settings['disable_logo'];
$cookie_policy_link = $settings['cookie_policy_link'];
$is_cookie_policy_url = $settings['is_cookie_policy_url'];
$cookie_policy_url = $settings['cookie_policy_url'];
$cookie_policy_page = $settings['cookie_policy_page'];
$showagain_tab = $settings['showagain_tab'];
$custom_css = $settings['custom_css'];
$notify_position_horizontal = $settings['notify_position_horizontal'];
$blocked_content_notify = $settings['blocked_content_notify'];
$wl = intval( $settings['wl'] );

$cookie_policy_html_inside_banner = "";
$cookie_policy_html_inside_popup = "";

$the_url = null;

if( $is_cookie_policy_url && $cookie_policy_url )
{
	$the_url = $cookie_policy_url;
}

if( !$is_cookie_policy_url && $cookie_policy_page )
{
	if( $is_wpml_enabled )
	{
		$the_url = get_permalink( icl_object_id( $cookie_policy_page, 'page', true) );
	}
	elseif( $is_polylang_enabled )
	{
		$the_url = get_permalink( pll_get_post( $cookie_policy_page ) );
	}
	else
	{
		$the_url = get_permalink( $cookie_policy_page );
	}
}

if( $the_url )
{
	$cookie_policy_html_inside_popup = '<a target="blank" href="'.esc_url( $the_url ).'">'.__('View the Cookie Policy', 'myagileprivacy').'</a>';
}

if( $cookie_policy_link && $the_url )
{
	$cookie_policy_html_inside_banner = $cookie_policy_html_inside_popup;
}

$spacing_text = "";
if( $cookie_policy_link && $cookie_policy_html_inside_banner )
{
	$spacing_text = " / ";
}

$show_again = $the_options["showagain_text"] ;

if( $reload_translation )
{
	$show_again = __('Manage consent', 'myagileprivacy');
}

$showagain_div_classes = "map_displayNone";

if( $disable_logo )
{
	$showagain_div_classes .= " nologo";
}

if( $showagain_tab == 0 )
{
	$showagain_div_classes .= " disactive";
}

if( $settings['with_css_effects'] == true )
{
	$showagain_div_classes .= " withEffects";
}

switch ($notify_position_horizontal) {
	case 'left':
		$showagain_div_classes .= " left_position";
		break;
	case 'right':
		$showagain_div_classes .= " right_position";
		break;
}

$notify_html .= '<div class="'.esc_attr( $showagain_div_classes ).'" id="' . esc_attr( $the_options["showagain_div_id"] ) . '" data-nosnippet="true"><a class="showConsent" href="">' . wp_kses( $show_again, MyAgilePrivacy::allowed_html_tags() ) . '</a>'.$spacing_text.$cookie_policy_html_inside_banner.'</div>';
echo wp_kses( $notify_html, MyAgilePrivacy::allowed_html_tags() );

$position_class_alt = ( $settings['is_bottom'] == false ) ? 'isBottom' : 'isTop';
$autoclose = ( $settings['blocked_content_notify_auto_shutdown'] == false ) ? '' : 'autoShutDown';
$composed_class_alt = implode( ' ', array( $position_class_alt, $autoclose, $with_css_effects_class ) );

$blocked_content_notification_html = "";

if( $blocked_content_notify )
{
	$blocked_content_notification_html = 	'<div class="map-blocked-content-notification-area '.esc_attr( $composed_class_alt ).'" id="map-blocked-content-notification-area" style="'.esc_attr( $composed_style ).'">
											<div class="map-area-container">
												<div class="map_notification-message">'.$blocked_content_text.'
													<div class="map_blocked_elems_desc"></div>
												</div>
											</div>
										</div>';
}

echo wp_kses( $blocked_content_notification_html, MyAgilePrivacy::allowed_html_tags() );
?>


<div class="map-modal" data-nosnippet="true" id="mapSettingsPopup" role="dialog" aria-labelledby="mapSettingsPopup" aria-hidden="true">
  <div class="map-modal-dialog" role="document">
	<div class="map-modal-content map-bar-popup <?php echo esc_attr( $with_css_effects_class );?>">
	  <button type="button" class="map-modal-close" id="mapModalClose">
		 <span class="map-modal-close-icon"></span>
		  <span class="map-sr-only"><?php echo esc_html__('Close', 'myagileprivacy');  ?></span>
	  </button>
	  <div class="map-modal-body">

	  	<div class="map-container-fluid map-tab-container">

			<div class="map-privacy-overview">
	            <h4><?php echo esc_html__('Privacy Setting', 'myagileprivacy'); ?></h4>
	        </div>

			<p><?php echo esc_html__('This website uses cookies to improve your experience while you navigate through the website.', 'myagileprivacy'); ?></p>


			<p><?php echo wp_kses( $cookie_policy_html_inside_popup, MyAgilePrivacy::allowed_html_tags() ) ; ?> </p>

			<?php

			foreach( $cookies_categories_data as $k => $v ):

				foreach( $v as $key => $value ):

					$the_remote_id = $value['remote_id'];

					$cleaned_cookie_name = str_replace( '"', '', $value['post_data']->post_title );

					$the_post_installation_type = isset( $value['post_meta']['_map_installation_type'][0] ) ? $value['post_meta']['_map_installation_type'][0] : 'js_noscript';
					$the_post_code = isset( $value['post_meta']['_map_code'][0] ) ? $value['post_meta']['_map_code'][0] : null;
					$the_noscript = isset( $value['post_meta']['_map_noscript'][0] ) ? $value['post_meta']['_map_noscript'][0] : null;
					$the_post_raw_code = isset( $value['post_meta']['_map_raw_code'][0] ) ? $value['post_meta']['_map_raw_code'][0] : null;
					$the_post_api_key = isset( $value['post_meta']['_map_api_key'][0] ) ? $value['post_meta']['_map_api_key'][0] : null;
			  ?>

				<div
				class="map-tab-section map_cookie_description_wrapper <?php if($k=='necessary'):?>_always_on<?php endif;?> <?php if($the_post_code || $the_post_raw_code):?> _with_code<?php endif;?>" data-cookie-baseindex="<?php echo esc_attr( $the_remote_id ); ?>"
				data-cookie-name="<?php echo esc_attr( $cleaned_cookie_name ); ?>"
				data-cookie-api-key="<?php echo esc_attr( $the_post_api_key ); ?>">
					<div class="map-tab-header <?php echo esc_attr( $with_css_effects_class );?>">
						<a role="button" class="map_expandItem map-nav-link map-settings-mobile" data-toggle="map-toggle-tab">
						<?php echo esc_html( $value['post_data']->post_title ); ?>
						</a>
						<?php if( $k == 'necessary' ):
						?>
							<span class="map-necessary-caption"><?php echo esc_html( $always_enable_text ) ?></span>
						<?php
						else:

		                    $map_switch = '<div class="map-switch">
		                        <input data-cookie-baseindex="'.esc_attr($the_remote_id).'" type="checkbox" id="map-checkbox-' . esc_attr($the_remote_id) . '" class="map-user-preference-checkbox MapDoNotTouch"  />
		                        <label for="map-checkbox-' . esc_attr($the_remote_id) . '" class="map-slider" data-map-enable="'.esc_attr($is_enabled_text).'" data-map-disable="'.esc_attr($is_disabled_text).'"><span class="map-sr-only">' . esc_html($value['post_data']->post_title) . '</span></label>
		                    </div>';

		                    echo wp_kses( $map_switch , MyAgilePrivacy::allowed_html_tags() ) ;
						?>

						<?php
						endif;
						?>
					</div>
					<div class="map-tab-content" style="display: none;">
						<div class="map-tab-pane map-fade">
							<?php
							echo wp_kses_post( $value['post_content'] ); ?>
						</div>
					</div>

					<?php

						if( $the_post_installation_type == 'js_noscript' &&
							$the_post_code  ):
					?>

						<script type="text/plain"
						class="my_agile_privacy_activate _js_noscript_type_mode"
						data-cookie-baseindex="<?php echo esc_attr($the_remote_id); ?>"
						data-cookie-name="<?php echo esc_attr( $cleaned_cookie_name ); ?>"
						data-cookie-api-key="<?php echo esc_attr( $the_post_api_key ); ?>">

							<?php

							echo strip_tags( stripslashes( $the_post_code ) );

							?>

						</script>


						<?php

							if( $the_noscript ):

						?>
							<noscript
							data-cookie-baseindex="<?php echo esc_attr($the_remote_id); ?>"
							data-cookie-name="<?php echo esc_attr( $cleaned_cookie_name ); ?>"
							data-cookie-api-key="<?php echo esc_attr( $the_post_api_key ); ?>"
							>

								<?php echo wp_kses_post( htmlspecialchars_decode( stripslashes( $the_noscript ), ENT_QUOTES ) ); ?>

							</noscript>

						<?php
							endif;
						?>


					<?php
						elseif( $the_post_installation_type == 'raw_code' &&
								$the_post_raw_code ) :
					?>

						<textarea
						style="display:none;"
						class="my_agile_privacy_activate _raw_type_mode"
						data-cookie-baseindex="<?php echo esc_attr($the_remote_id); ?>"
						data-cookie-name="<?php echo esc_attr( $cleaned_cookie_name ); ?>"
						data-cookie-api-key="<?php echo esc_attr( $the_post_api_key ); ?>">


							<?php echo htmlspecialchars_decode( stripslashes( $the_post_raw_code ), ENT_QUOTES ); ?>

						</textarea>

					<?php
						endif;
					?>

			  </div>

			<?php

				endforeach;
			endforeach;

			?>


		</div> <!-- map-container-fluid -->

		<?php
		if( $wl == 0 ):

			if( ( isset( $rconfig ) &&
				$rconfig['credits_image_only'] == 1 ) ):
		?>

				<div class="modal_credits">
					<?php if( $the_options['pa'] == 1 ): ?>
						<img src="<?php echo plugin_dir_url( __DIR__ );?>img/privacy-by-pro.png" alt="Privacy by My Agile Privacy">
					<?php else: ?>
						<img src="<?php echo plugin_dir_url( __DIR__ );?>img/privacy-by-basic.png" alt="Privacy by My Agile Privacy">
					<?php endif; ?>
				</div>

			<?php
			else:
			?>

				<div class="modal_credits">
					<?php if( $the_options['pa'] == 1 ): ?>
						<a href="https://www.myagileprivacy.com/about/?utm_source=referral&utm_medium=plugin-pro&utm_campaign=customize" target="_blank" rel="nofollow"><img src="<?php echo plugin_dir_url( __DIR__ );?>img/privacy-by-pro.png" alt="Privacy by My Agile Privacy"></a>
					<?php else: ?>
						<a href="https://www.myagileprivacy.com/about/?utm_source=referral&utm_medium=plugin-basic&utm_campaign=customize" target="_blank" rel="nofollow"><img src="<?php echo plugin_dir_url( __DIR__ );?>img/privacy-by-basic.png" alt="Privacy by My Agile Privacy"></a>
					<?php endif; ?>
				</div>

			<?php
			endif;
			?>


		<?php
		else:
		?>
			<br>
		<?php
		endif;
		?>


	</div> <!-- map-modal-body -->
	</div>
  </div>
</div>
<div class="map-modal-backdrop map-fade map-settings-overlay"></div>
<div class="map-modal-backdrop map-fade map-popupbar-overlay"></div>

<?php

if( $custom_css )
{
	echo '<style type="text/css">'.esc_attr( $custom_css ).'</style>';
}

// enable indexing
echo "<!--googleon: all-->";
